#==============================================================================
# package      DE_EPAGES::AmazonPayments::UI::PaymentMethodAmazonPayments
# state        private
#==============================================================================
package DE_EPAGES::AmazonPayments::UI::PaymentMethodAmazonPayments;
use base DE_EPAGES::Payment::UI::PaymentMethod;
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use strict;

use DE_EPAGES::AmazonPayments::API::Constants qw (
    AMAZON_REGIONS
    AMAZON_BUTTON_COLORS
    CREDENTIALS_OK
    CREDENTIALS_UNAUTHORIZED_DEVELOPER
    CREDENTIALS_UNKNOWN_MERCHANT_ID
    AMAZON_DE_LANGUAGECODE
    AMAZON_UK_LANGUAGECODE
);
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Object::API::Table::Language qw( GetInfoLanguagesByCode2 );
use DE_EPAGES::Presentation::API::Constants qw ( PASSWORD_DUMMY );

#==============================================================================
sub ViewSettings {
    my $self = shift;
    my ($Servlet) = @_;
    my $PaymentMethod = $Servlet->object;

    my $SelectedRegion = $PaymentMethod->get('AmazonRegion');

    $Servlet->vars('NoWebServerScriptNamePartError', 0);
    $Servlet->vars('NoValidSSLCertificateError', 0);
    my $CallbackUrl;
    eval {
        $CallbackUrl = $PaymentMethod->callbackUrl();
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        if ( $Error->code() eq 'NoWebServerScriptNamePartError' ) {
            $Servlet->vars('NoWebServerScriptNamePartError', 1);
        } elsif ( $Error->code() eq 'NoValidSSLCertificateError') {
              $Servlet->vars('NoValidSSLCertificateError', 1);
        } elsif ( $Error->code() eq 'UrlBuildingError') {
            $Servlet->vars('CallBackUrlBuildingError', 1);
        } else {
            Error($Error);
        }
    }
    if(defined $SelectedRegion) {
        $self->_loadLinks($Servlet, $SelectedRegion);
        $Servlet->vars('SelectedRegion', AMAZON_REGIONS->{$SelectedRegion});
    }
    $Servlet->vars('Regions', $PaymentMethod->regions);
    $Servlet->vars('ButtonColors', AMAZON_BUTTON_COLORS);
    $Servlet->vars('CallbackURL', $CallbackUrl);
    return;
}

sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;
    my $PaymentMethod = $Servlet->object;
    my $Form = $Servlet->form;

    if(! $PaymentMethod->get('AmazonRegion')) {
        my $hValues = $Form->form($PaymentMethod, 'AmazonRegion');
        $Form->executeFormError({
            'Reason'=> 'UNKNOWN_REGION',
            'Name'  => 'AmazonRegion',
        }) unless exists AMAZON_REGIONS->{$hValues->{'AmazonRegion'}};
        $PaymentMethod->set($hValues);
    } else {
        my $hValues = $Form->form($PaymentMethod, 'AmazonPaymentsSettings');

        $hValues->{'SecretKey'} = $PaymentMethod->get('SecretKey')
            if ( defined $hValues->{'SecretKey'} && $hValues->{'SecretKey'} eq PASSWORD_DUMMY );

        $Form->executeFormError({
            'Reason'=> 'UNKNOWN_COLOR',
            'Name'  => 'ButtonColor',
        }) if(defined($hValues->{'ButtonColor'}) && !($hValues->{'ButtonColor'} ~~ AMAZON_BUTTON_COLORS));
        delete $hValues->{'IsInTestMode'} unless defined $hValues->{'IsInTestMode'};
        $PaymentMethod->set($hValues);
        my $Checked = $self->_doCredentialsCheck($Servlet);
        $PaymentMethod->set({ 'CredentialsChecked' => $Checked });
        $Servlet->vars('CredentialsCheckSucceeded', $Checked);
    }
}

sub _doCredentialsCheck {
    my $self = shift;
    my ($Servlet) = @_;
    my $PaymentMethod = $Servlet->object;
    TestObject('PaymentMethod', $PaymentMethod, 'DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments');

    my $Status = undef;
    eval {
        $Status = $PaymentMethod->client->check();
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) if ($Error->code eq 'NativeDie');
        LogError($Error);
        $Status = 'UNKNOWN_ERROR';
    }
    return 1 if ($Status eq CREDENTIALS_OK);

    my $Errors = {
        (CREDENTIALS_UNKNOWN_MERCHANT_ID) => {
            Reason => CREDENTIALS_UNKNOWN_MERCHANT_ID,
            Name   => 'MerchantID',
        },
        (CREDENTIALS_UNAUTHORIZED_DEVELOPER) => {
            Reason => CREDENTIALS_UNAUTHORIZED_DEVELOPER,
            Name   => 'MerchantID',
        },
        UNKNOWN_ERROR => {
            Reason => 'UNKNOWN_ERROR',
            Name   => 'UNKNOWN_ERROR'
        }
    };

    # this terminates the viewaction.
    $Servlet->form->executeFormError($Errors->{$Status} // $Errors->{'UNKNOWN_ERROR'});
}

sub _loadLinks {
    my $self = shift;
    my ($Servlet, $SelectedRegion) = @_;
    my $LanguageCode = $SelectedRegion eq 'DE' ? AMAZON_DE_LANGUAGECODE : AMAZON_UK_LANGUAGECODE;
    my $LanguageID = GetInfoLanguagesByCode2($LanguageCode)->[0]->{'LanguageID'};

    my $RootObject = LoadRootObject();

    my $RegisterUrl = $RootObject->get('AmazonPayments_RegisterUrl', $LanguageID);
    my $InfoUrl = $RootObject->get('AmazonPayments_InfoUrl', $LanguageID);
    my $AdminUrl = $RootObject->get('AmazonPayments_AdminUrl', $LanguageID);
    my $ActivateMwsUrl = $RootObject->get('AmazonPayments_ActivateMwsUrl', $LanguageID);

    $Servlet->vars('REGISTER_URL', $RegisterUrl);
    $Servlet->vars('INFO_URL', $InfoUrl);
    $Servlet->vars('ADMIN_URL', $AdminUrl);
    $Servlet->vars('ACTIVATE_MWS_URL', $ActivateMwsUrl);
}


1;
