#========================================================================================
# package      DE_EPAGES::AppStore::API::Shop
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides shop functions of AppStore.
#========================================================================================
package DE_EPAGES::AppStore::API::Shop;
use base Exporter;
use strict;

use DE_EPAGES::AppStore::API::Constants qw (
    UNITY_SCRIPT_TAGS_APP_NAME
    MAX_TRIES_FOR_INITIAL_UNITY_SCRIPT_TAG
);
use DE_EPAGES::AppStore::API::Object::ScriptTagAppClient;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use HTTP::Status qw( HTTP_NOT_FOUND );

our @EXPORT_OK = qw (
    CreateUnityScriptTagsApp
    DeleteUnityScriptTagsApp
    GetUnityScriptTagsApp
);

sub ErrorDefinition {
    return {
        'AppNotInstalled' => 'App \'#Name\' not deleted because it is not installed'
    };
}

#========================================================================================
# function     GetUnityScriptTagsApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetUnityScriptTagsApp($Shop);
#----------------------------------------------------------------------------------------
# description  Get the app used to hand over script-tags to Unity (epages-ui)
#               for the given shop.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $ScriptTagAppClient | script-tags app client | object
#========================================================================================
sub GetUnityScriptTagsApp {
    my ($Shop) = @_;
    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');

    my $UnityScriptTagsAppGUID = $Shop->get('UnityScriptTagsAppGUID');

    my $ScriptTagAppClient;
    if (defined $UnityScriptTagsAppGUID) {
        # get credentials from existing app
        $ScriptTagAppClient = DE_EPAGES::AppStore::API::Object::ScriptTagAppClient->create({'Shop' => $Shop});
    } else {
        # create new app, if there has not been any so far
        $ScriptTagAppClient = CreateUnityScriptTagsApp($Shop);
    }

    return $ScriptTagAppClient;
}

#========================================================================================
# function     CreateUnityScriptTagsApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       CreateUnityScriptTagsApp($Shop);
#----------------------------------------------------------------------------------------
# description  Creates a new private developer app used to hand over script-tags to
#               Unity (epages-ui).
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $ScriptTagAppClient | script-tags app client | object
#========================================================================================
sub CreateUnityScriptTagsApp {
    my ($Shop) = @_;
    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');

    my $ScriptTagAppClient = DE_EPAGES::AppStore::API::Object::ScriptTagAppClient->create({'Shop' => $Shop});
    my $UnityScriptTagsApp;
    my $AppAlreadyExists = 0;

    my $UnityScriptTagsAppGUID = $Shop->get('UnityScriptTagsAppGUID');
    if (defined $UnityScriptTagsAppGUID) {
        eval {
            # try to get existing app
            $UnityScriptTagsApp = $ScriptTagAppClient->getApp();
        };
        if (ExistsError()) {
            my $Error = GetError();
            # ignore error '404 not found' and create a new app instead
            Error($Error) unless $Error->vars->{'Status'} == HTTP_NOT_FOUND;
        } else {
            $AppAlreadyExists = 1;
        }
    }

    # create a new app if it does not exist
    $UnityScriptTagsApp =$ScriptTagAppClient->createApp() unless $AppAlreadyExists;

    my $aScriptTagURLs = [];

    TriggerHook( 'API_RegisterInitialScriptTags',
        { 'Shop' => $Shop, 'ScriptTagUrls' => $aScriptTagURLs } );

    foreach my $ScriptTagURL (@$aScriptTagURLs) {
        for (my $Try = 1; $Try <= MAX_TRIES_FOR_INITIAL_UNITY_SCRIPT_TAG; $Try++) {
            # retry adding a script tag after an error due to UNITY-5786
            eval {
                # add initial script-tags (does nothing if ScriptTagURL is already added
                $ScriptTagAppClient->addScriptTag($ScriptTagURL);
            };
            if (ExistsError()) {
                my $Error = GetError();
                if ($Try == MAX_TRIES_FOR_INITIAL_UNITY_SCRIPT_TAG) {
                    # only throw an error in case all tries have failed
                    Error($Error);
                } else {
                    # otherwise wait a second and try again
                    Warning($Error);
                    sleep 1;
                }
            } else {
                last;
            }
        }
    }

    return $ScriptTagAppClient;
}

#========================================================================================
# function     DeleteUnityScriptTagsApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteUnityScriptTagsApp($Shop);
#----------------------------------------------------------------------------------------
# description  Deletes the Unity script-tags app for the given shop.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $ScriptTagAppClient | script-tags app client | object
#========================================================================================
sub DeleteUnityScriptTagsApp {
    my ($Shop) = @_;
    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');

    my $UnityScriptTagsAppGUID = $Shop->get('UnityScriptTagsAppGUID');
    # withput ID we do not know which app to delete
    Error('AppNotInstalled', {'Name' => UNITY_SCRIPT_TAGS_APP_NAME}) unless defined $UnityScriptTagsAppGUID;

    my $ScriptTagAppClient = DE_EPAGES::AppStore::API::Object::ScriptTagAppClient->create({'Shop' => $Shop});

    $ScriptTagAppClient->deleteApp();

    return $ScriptTagAppClient;
}

1;
