#========================================================================================
# package      DE_EPAGES::MediaGallery::Hooks::Shop
# state        public
# description  Adds the MediaGallery as a child object to the shop.
#========================================================================================
package DE_EPAGES::MediaGallery::Hooks::Shop;

use strict;
use DE_EPAGES::Object::API::Factory qw (LoadObject);
use DE_EPAGES::Core::API::File qw (CreateDirectory);
use DE_EPAGES::MediaGallery::API::Cartridge;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

#========================================================================================
# function     OnInsertShop
#----------------------------------------------------------------------------------------
# syntax       OnInsertShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Adds the MediaGallery as a child object to the shop.
#               (Hook OBJ_InsertShop)
#----------------------------------------------------------------------------------------
# input        $hParams |
#               <ul>
#                   <li>Object - shop - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};

    my $Cartridge = DE_EPAGES::MediaGallery::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/DefaultShop.xml', $Shop->pathString);

    # create directory for MediaGallery
    my $MediaGallery = $Shop->child('MediaGallery');
    CreateDirectory($MediaGallery->get('PublicPath'));

    # EPG-14379: set feature GallerySize dirty to force re-calculation after import of demo content
    $Shop->featureIsDirty( 'GallerySize', 1 );
    return;
}

#========================================================================================
# function     OnInsertShopFeature
#----------------------------------------------------------------------------------------
# syntax       OnInsertShopFeature( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook API_InsertShopFeature.
#               Imports MediaGallery "UnityMediaImportFiles" into a new Unity shop
#               if the feature 'Unity' is set.
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                 <li>ShopID     - shop id    - int
#                 <li>FeatureID  - feature id - int
#               </ul>
#========================================================================================
sub OnInsertShopFeature {
    my ($hParams) = @_;
    my $ShopID = $hParams->{'ShopID'};
    my $FeatureID = $hParams->{'FeatureID'};
    my $Feature = LoadObject ( $FeatureID );
    if ($Feature->alias eq 'Unity') {
        my $Shop = LoadObject( $ShopID );
        my $Unity = $Shop->featureMaxValue('Unity');
        my $Cartridge = DE_EPAGES::MediaGallery::API::Cartridge->new;
        ImportObjects($Cartridge->xmlDirectory().'/DefaultShopUnity.xml', $Shop->pathString) if $Unity;
    }
}

#========================================================================================
# function     OnAfterUpdateShopFeature
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateShopFeature( $hParams );
#----------------------------------------------------------------------------------------
# description  OnAfterUpdateShopFeature is triggered,
#               Imports MediaGallery "UnityMediaImportFiles" for Unity shop
#----------------------------------------------------------------------------------------
# input        $hShopFeature - feature object, used keys are
#               <ul>
#                   <li>FeatureID - feature id            | integer</li>
#                   <li>ShopID    - shop id               | integer</li>
#                   <li>MaxValue  - maximum feature value | int</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateShopFeature {
    my ($hShopFeature) = @_;

    my $Feature = LoadObject($hShopFeature->{'FeatureID'});
    return unless ($Feature->alias eq 'Unity' and $hShopFeature->{'MaxValue'});

    my $Shop = LoadObject($hShopFeature->{'ShopID'});
    my $Cartridge = DE_EPAGES::MediaGallery::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/DefaultShopUnity.xml', $Shop->pathString) unless $Shop->existsChild('UnityMediaImportFiles');
    return;
}

1;
