#========================================================================================
# package      DE_EPAGES::MolliePayments::API::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  Tooling for specific tasks like log filtering
#========================================================================================
package DE_EPAGES::MolliePayments::API::Tools;
use base qw( Exporter );
use DE_EPAGES::Core::API::PerlTools qw( isValid );

use DE_EPAGES::Core::API::Object::NumberFormat;
use DE_EPAGES::MolliePayments::API::Constants qw ( SUPPORTED_LOCALES DEFAULT_LOCALE );

use strict;

our @EXPORT_OK = qw (
    GetSupportedLocale
    FormatFloatForMollie
    FilterTokensResponse
    FilterOnboardingReturnQuery
    ParseErrorStringToHash
);

#========================================================================================
# function     FormatFloatForMollie
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FormattedFloatNumber = FormatFloatForMollie($FloatNumber);
#----------------------------------------------------------------------------------------
# description  It formats float number according to Mollie API V2
#----------------------------------------------------------------------------------------
# input        $FloatNumber | float number | float
# return       $FormattedFloatNumber | formatted float number string | string
#========================================================================================
sub FormatFloatForMollie {
    my ($FloatNumber) = @_;

    my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new(
        'decimal_digits' => 2, 'decimal_fill' => 2, decimal_point  => '.', thousands_sep => ''
    );
    return $Formatter->format_number($FloatNumber);
}

#========================================================================================
# function     GetSupportedLocale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SupportedLocale = GetSupportedLocale($Locale);
#----------------------------------------------------------------------------------------
# description  It checks if provided locale is supported and returns it if so.
#               If not supported, then the most appropriated one is provided.
#----------------------------------------------------------------------------------------
# input        $Locale | locale string | string
# return       $SupportedLocale | locale string | string
#========================================================================================
sub GetSupportedLocale {
    my ($Locale) = @_;

    if ($Locale ~~ SUPPORTED_LOCALES) {
        return $Locale;
    }

    my ($Language, $Country) = split(/_/, $Locale);
    my $LanguageLocale = $Language . '_' . uc($Language);
    if ($LanguageLocale ~~ SUPPORTED_LOCALES) {
        return $LanguageLocale;
    }

    my $CountryLocale = lc($Country) . '_' . $Country;
    if ($CountryLocale ~~ SUPPORTED_LOCALES) {
        return $CountryLocale;
    }

    return DEFAULT_LOCALE;
}

#========================================================================================
# function     FilterTokensResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       FilterTokensResponse($LogString);
#----------------------------------------------------------------------------------------
# description  It filters token codes (to be used as paramter into LogPayment calls)
#----------------------------------------------------------------------------------------
# input        $LogString | string to filter | string
#========================================================================================
sub FilterTokensResponse {
    my $LogString = shift;

    $LogString =~ s/"clientId":"\w+"/"clientId":"****"/gix;
    $LogString =~ s/"clientSecret":"\w+"/"clientSecret":"****"/gix;
    $LogString =~ s/"code":"\w+"/"code":"****"/gix;
}

#========================================================================================
# function     FilterOnboardingReturnQuery
# state        public
#----------------------------------------------------------------------------------------
# syntax       FilterOnboardingReturnQuery($LogString);
#----------------------------------------------------------------------------------------
# description  It filters authorizatio code from URL query (to be used as paramter into LogPayment calls)
#----------------------------------------------------------------------------------------
# input        $LogString | string to filter | string
#========================================================================================
sub FilterOnboardingReturnQuery {
    my $LogString = shift;

    $LogString =~ s/'auth_\w+'/'auth_---replaced---'/gix;
}

#========================================================================================
# function     ParseErrorStringToHash
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hErrorResponse = ParseErrorStringToHash($ErrorString, $StatusCode);
#----------------------------------------------------------------------------------------
# description  Convert mollie error response string to a hash
#               String should contain  "MollieErrorResponse"
#               Split the error message in 2 parts: exception name, fields
#               Trim both ends [ ] of the fields part
#               Split fields by ' | '
#               Split key value pairs by =
#----------------------------------------------------------------------------------------
# input        $ErrorString | Mollie error string | string
#               $StatusCode | HTTP status code | string
# output       $hMollieResponse |                 | hash
# return       $hMollieResponse | Mollie response | hash
#========================================================================================
sub ParseErrorStringToHash {
    my ($ErrorString, $StatusCode) = @_;

    return unless isValid($ErrorString);
    return unless (index($ErrorString, "MollieErrorResponse") != -1);

    my @Words = split(/\s/, $ErrorString, 2);
    my $ExceptionName = $Words[0];
    my $ContentString = $Words[1];
    $ContentString =~ s/^\[+|\]+$//g;
    my @ContentWords = split(/\s\|\s/, $ContentString);

    my %hContent =  map { split( /=/, $_, 2 ) } @ContentWords;
    my $hErrorResponse = {
        'exception' => $ExceptionName,
        'status'    => $StatusCode,
        'content'   => \%hContent
    };

    return $hErrorResponse;
}

1;
