#========================================================================================
# package      DE_EPAGES::MolliePayments::UI::BasketOrder
# base         DE_EPAGES::Presentation::UI::Object
# state        private
#----------------------------------------------------------------------------------------
# description  UI Controller for basket or order
#========================================================================================
package DE_EPAGES::MolliePayments::UI::BasketOrder;
use base DE_EPAGES::Presentation::UI::Object;

use DE_EPAGES::Core::API::PerlTools qw ( isValid GetCurrentDateTime );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;
use DE_EPAGES::MolliePayments::API::PaymentStatusService;
use DE_EPAGES::MolliePayments::API::Constants qw (
    PAYMENT_TYPE_MOLLIE_PAYMENTS
    MOLLIE_PAYMENT_STATUS_ERRORS
    PAYMENT_STATUS_FAILED
    PAYMENT_STATUS_CANCELED
    PAYMENT_STATUS_EXPIRED
);
use strict;


#========================================================================================
# function     ViewPaymentMolliePayments
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewPaymentMolliePayments( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller for classical/mobile checkout
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewPaymentMolliePayments {
    my $self    = shift;
    my ($Servlet) = @_;

    my $Object  = $Servlet->object();
    my $LineItemPayment = $Object->get('LineItemContainer')->get('Payment');
    my $PaymentMethod   = $LineItemPayment->get('PaymentMethod');
    my $GUID = $LineItemPayment->get('GUID');

    $LineItemPayment->set( {
        'IsTestTransaction' => $PaymentMethod->get('IsInTestMode'),
        'TransID' => $GUID,
        'TransTime' => GetCurrentDateTime(),
     });

    return;
}

#========================================================================================
# function     ViewMultiPaymentMolliePayments
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewMultiPaymentMolliePayments( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller for multi-step checkout
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewMultiPaymentMolliePayments {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess', $BasketOrderProcess->tleHash());

    $self->ViewPaymentMolliePayments($Servlet);

    return;
}

sub ValidateOrderPaymentStatus {
    my $self = shift;
    my ($Servlet, $LineItemPayment) = @_;

    my $PaymentStatusService = $self->_createPaymentStatusService();

    my $TransactionStatus = $LineItemPayment->get('TransStatus');

    if (! isValid($TransactionStatus)) {

        $self->_LogPayment('Mollie ValidateOrderPaymentStatus is not valid', { 'TransactionStatus' => $TransactionStatus});
        eval {
            $TransactionStatus = $PaymentStatusService->getMollieOrderAndHandleStatus($LineItemPayment);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $self->_redirectToPaymentSelection($Error->message(), $Servlet);
        }
    }

    $self->_LogPayment('Mollie ValidateOrderPaymentStatus', { 'NewStatus' => $TransactionStatus });
    if( $TransactionStatus ~~ MOLLIE_PAYMENT_STATUS_ERRORS ) {
        $PaymentStatusService->deleteLineItemPaymentStatusInfo($LineItemPayment);
        $self->_redirectToPaymentSelection($TransactionStatus, $Servlet);
    }
}

sub _getFailReason {
    my $self = shift;
    my ($PaymentStatus) = @_;

    if ($PaymentStatus eq PAYMENT_STATUS_FAILED ) {
        return 'card_declined';
    } elsif ($PaymentStatus eq PAYMENT_STATUS_CANCELED) {
        return 'card_declined';
    } elsif ($PaymentStatus eq PAYMENT_STATUS_EXPIRED) {
        return 'card_declined';
    }
    return 'unknown_error_code';
}

sub _createPaymentStatusService {
    my $self = shift;

    return DE_EPAGES::MolliePayments::API::PaymentStatusService->new();
}

sub _redirectToPaymentSelection {
    my $self = shift;
    my ($StatusMessage, $Servlet) = @_;

    $Servlet->vars('ViewAction', 'ViewMultiCheckoutPayment');

    my $FailReason = $self->_getFailReason($StatusMessage);
    $self->_LogPayment('Mollie ValidateOrderPaymentStatus redirect to payment',{'StatusMessage' => $StatusMessage, 'FailReason' => $FailReason });
    $Servlet->form()->executeFormError({
        'Name' => 'MolliePaymentsFailed',
        'Reason' => $FailReason
    });
    return;
}

sub _LogPayment {
    my $self = shift;
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, $Name, $DebugData, $Section, $Filter);
}


1;