#========================================================================================
# package      DE_EPAGES::MolliePayments::UI::JSON::BasketOrder
# base         DE_EPAGES::Presentation::UI::JSON
# state        private
#----------------------------------------------------------------------------------------
# description  UI Controller for basket or order
#========================================================================================
package DE_EPAGES::MolliePayments::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;

use DE_EPAGES::MolliePayments::API::Constants qw ( PAYMENT_TYPE_MOLLIE_PAYMENTS PROFILES_LIMIT_MOLLIE_PAYMENTS );
use DE_EPAGES::MolliePayments::API::PaymentStatusService;
use DE_EPAGES::MolliePayments::API::MolliePaymentsService;
use DE_EPAGES::MolliePayments::API::Requests::Methods;
use DE_EPAGES::MolliePayments::API::Requests::Orders;
use DE_EPAGES::MolliePayments::API::Requests::OrdersShipments;
use DE_EPAGES::MolliePayments::API::Requests::Profiles;
use DE_EPAGES::MolliePayments::API::Tools qw ( ParseErrorStringToHash );
use DE_EPAGES::MolliePayments::API::Requests::Tokens;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw (LogError);
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::Presentation::API::DereferrerToken qw ( GenerateDereferrerToken );

use feature 'switch';

use strict;

#========================================================================================
# function     GetMolliePaymentsProfiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetMolliePaymentsProfiles( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller for GetMolliePaymentsProfiles at MBO ViewSettings
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub GetMolliePaymentsProfiles {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();

    my $ProfileHelper = $self->_createProfilesHandler({
        'payment' => $PaymentMethod
    });
    my $ProfileId = $Servlet->params()->{'ProfileId'};
    $ProfileHelper->withFrom($ProfileId)->withLimit(PROFILES_LIMIT_MOLLIE_PAYMENTS);

    my $MollieService = $self->_createMolliePaymentsService($ProfileHelper);

    my $hProfilesResponse = {};

    eval {
        $hProfilesResponse = $MollieService->doGetRequest();
    };

    if (ExistsError()) {
        my $Error = GetError();
        $self->_LogPayment(
            'Error when getting profiles', $Error->message()
        );
        $Servlet->vars('MollieConnectionError', 1);
        $self->_writeContentOutput($hProfilesResponse);

        return;
    }

    $self->_LogPayment(
        'Get Profiles',
        { 'PaymentMethod' =>  $PaymentMethod->id(), 'Profiles Response' => $hProfilesResponse },undef , undef
    );

    $self->_writeContentOutput($hProfilesResponse);
}

#========================================================================================
# function     GetMolliePaymentsMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetMolliePaymentsMethods( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller for GetMolliePaymentsMethods at MBO ViewSettings
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub GetMolliePaymentsMethods {
    my $self = shift;
    my ($Servlet) = @_;

    my $IsInTestMode = $Servlet->params()->{'IsInTestMode'};
    my $ProfileId = $Servlet->params()->{'ProfileId'};

    my $PaymentMethod = $Servlet->object();
    my $hMethodsData = {
        'payment' => $PaymentMethod
    };
    my $MethodsHandler = $self->_createMethodsHandler($hMethodsData);
    $MethodsHandler->withProfileId($ProfileId);
    my $MollieService = $self->_createMolliePaymentsService($MethodsHandler);

    my $hLogParams = {
        'PaymentMethodId' =>  $PaymentMethod->id(),
        'profileId' => $ProfileId,
    };
    my $hMethodsResponse = {};

    eval {
        $hMethodsResponse = $MollieService->withTestMode($IsInTestMode)->doGetRequest();
    };
    if (ExistsError()) {
        my $Error = GetError();
        $hLogParams->{'Error'} = $Error->message();
        $self->_LogPayment('Error when getting Payment Methods',$hLogParams, undef, undef);
        $Servlet->vars('MollieConnectionError', 1);
        $self->_writeContentOutput($hMethodsResponse);
        return;
    }

    $hLogParams->{'MethodsResponse'} = $hMethodsResponse;
    $self->_LogPayment('Get Payment Methods response', $hLogParams, undef , undef);

    $self->_writeContentOutput($hMethodsResponse);
}


#========================================================================================
# function     OrderCreationMolliePayments
# state        public
#----------------------------------------------------------------------------------------
# syntax       OrderCreationMolliePayments( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller for mollie order creation on checkout
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub OrderCreationMolliePayments {
    my $self = shift;
    my ($Servlet) = @_;

    my $Service = $self->_buildOrdersService($Servlet);
    my $hResponse;

    eval {
        $hResponse = $Service->doPostRequest();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        my $hErrorData = $self->_formatErrorResponse('OrderCreation', $Error, $hResponse);

        $self->_LogPayment('Mollie order generation response error', $hErrorData);
        $self->_writeContentOutput($hErrorData);
    } else {
        $self->_LogPayment('Mollie order generation response success', $hResponse);
        my $Basket = $Servlet->object();
        my $LineItemPayment = $Basket->get('LineItemContainer')->get('Payment');
        $LineItemPayment->set({ 'MollieOrderId' => $hResponse->{'id'} });
        $self->_writeContentOutput($hResponse->{'checkout'});
    }
}

#========================================================================================
# function     OrderShipmentMolliePayments
# state        public
#----------------------------------------------------------------------------------------
# syntax       OrderShipmentMolliePayments( $Order );
#----------------------------------------------------------------------------------------
# description  Controller for mollie order shipment on MBO order details page
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub OrderShipmentMolliePayments {
    my $self = shift;
    my ($Servlet) = @_;

    my $CustomerOrder = $Servlet->object();
    my $LineItemContainer = $CustomerOrder->get('LineItemContainer');
    my $LineItemPayment = $LineItemContainer->get('Payment');

    my $Service = $self->_buildOrdersShipmentsService($LineItemContainer, $LineItemPayment);
    my $hResponse;

    eval {
        $hResponse = $Service->doPostRequest();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        my $hErrorData = $self->_formatErrorResponse('OrderShipmentCreation', $Error, $hResponse);
        $self->_LogPayment('Mollie order shipment generation response error', $hErrorData);
        my $hErrorResponse = ParseErrorStringToHash($Error->vars->{'Response'}, $Error->vars->{'Status'});
        my $ErrorMessage = 'Unknown';
        if (defined($hErrorResponse->{'content'}) && defined($hErrorResponse->{'content'}->{'detail'})) {
            $ErrorMessage = $hErrorResponse->{'content'}->{'detail'};
        }
        $self->_writeContentOutput({
            'StatusCode' => 'ERROR',
            'Message' => $ErrorMessage
        });
    } else {
        $self->_LogPayment('Mollie order shipment generation response success', $hResponse);
        $self->_handleStatusChange($LineItemPayment);
        my $SuccessMessage = 'Shipment Id ' . $hResponse->{'id'} . ' created';
        $self->_writeContentOutput({
            'StatusCode' => 'OK',
            'Message'    => $SuccessMessage
        });
    }
}

#========================================================================================
# private methods
#========================================================================================
sub _handleStatusChange {
    my $self = shift;
    my ($LineItemPayment) = @_;

    
    my $PaymentStatusService = $self->_createPaymentStatusService();

    eval {
        $PaymentStatusService->getMollieOrderAndHandleStatus($LineItemPayment);
    };

    if (ExistsError()) {
        my $Error = GetError();
        $self->_LogPayment('Error on OrderShipmentMolliePayments->_handleStatusChange', {'Error' => $Error->message()});
    }
    return;
}
#----------------------------------------------------------------------------------------
sub _createPaymentStatusService {
    my $self = shift;
    return DE_EPAGES::MolliePayments::API::PaymentStatusService->new();
}
#----------------------------------------------------------------------------------------
sub _createProfilesHandler {
    my $self = shift;
    my ($hData) = @_;

    return DE_EPAGES::MolliePayments::API::Requests::Profiles->new($hData);
}

#----------------------------------------------------------------------------------------
sub _createOrdersHandler {
    my $self = shift;
    my ($hOrdersData) = @_;

    return DE_EPAGES::MolliePayments::API::Requests::Orders->new($hOrdersData);
}
#----------------------------------------------------------------------------------------
sub _createMethodsHandler {
    my $self = shift;
    my ($hMethodsData) = @_;

    return DE_EPAGES::MolliePayments::API::Requests::Methods->new($hMethodsData);
}

#----------------------------------------------------------------------------------------
sub _createMolliePaymentsService {
    my $self = shift;
    my ($HandlerObject) = @_;

    return DE_EPAGES::MolliePayments::API::MolliePaymentsService->new($HandlerObject);
}

#----------------------------------------------------------------------------------------
sub _buildOrdersService {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object();
    TestObject('Basket', $Basket, 'DE_EPAGES::Order::API::Object::Basket');
    my $LineItemContainer = $Basket->get('LineItemContainer');

    my $LineItemPayment = $Basket->get('LineItemContainer')->get('Payment');

    my $OrderRedirectUrl = $self->_buildOrderRedirectUrl($Servlet, $Basket);
    my $PaymentWebhookUrl = $self->_buildWebhookURL(
        $Servlet,
        $Basket,
        $LineItemPayment,
        {'ChangeAction' => 'MolliePaymentsPaymentWebhook'}
    );

    my $OrdersHandler =  $self->_createOrdersHandler({
        'redirectUrl' => $OrderRedirectUrl,
        'paymentWebhookUrl' => $PaymentWebhookUrl,
        'basket' => $Basket,
        'payment' => $LineItemPayment->get('PaymentMethod'),
        'container' => $LineItemContainer,
    });
    $self->_LogPayment('Mollie order generation request', $OrdersHandler->getPayload('POST'));

    return $self->_createMolliePaymentsService($OrdersHandler);
}

#----------------------------------------------------------------------------------------
sub _buildOrderRedirectUrl {
    my $self = shift;
    my ($Servlet, $Basket) = @_;

        my $hParams = {
            'ChangeAction' => 'SaveMultiCheckoutConfirmation',
        },
        my $hOptions = {
        'Type' => 'sf',
        'AddToken' => '1',
        'UseSSL' => '1',
    };

    return $self->_BuildShopUrl($Basket, $hParams, $hOptions, $Servlet);
}
#----------------------------------------------------------------------------------------
sub _buildWebhookURL {
    my $self = shift;
    my ( $Servlet, $Basket, $LineItemPayment, $hParams ) = @_;

    my $Shop = $Basket->get('Shop');

    my $LineItemPaymentId = $LineItemPayment->guid();

    $hParams->{'LineItemPaymentGUID'} = $LineItemPaymentId;
    my $hOptions = {
        'Type' => 'sf',
        'UseSSL' => 1,
        'AddToken' => 1
    };

    my $URL = $self->_BuildShopUrl( $Shop, $hParams, $hOptions, $Servlet );

    my $PayLoad = sprintf('%s&%s&%s', $Shop->alias(), $Shop->id(), $LineItemPaymentId);
    my $DereferrerToken = $self->_GenerateDereferrerToken($PayLoad);

    return sprintf('%s&DereferrerToken=%s', $URL, $DereferrerToken);
}
#----------------------------------------------------------------------------------------
sub _GenerateDereferrerToken {
    my $self = shift;
    my ($PayLoad) = @_;
    
    return GenerateDereferrerToken($PayLoad);
}
#----------------------------------------------------------------------------------------
sub _BuildShopUrl {
    my $self = shift;
    my ($Object, $hParams, $hOptions, $Servlet) = @_;
    
    return BuildShopUrl($Object, $hParams, $hOptions, $Servlet);
}
#----------------------------------------------------------------------------------------
sub _buildOrdersShipmentsService {
    my $self = shift;
    my ($LineItemContainer, $LineItemPayment) = @_;

    my $Payment = $LineItemPayment->get('PaymentMethod');
    my $OrdersShipmentsHandler =  DE_EPAGES::MolliePayments::API::Requests::OrdersShipments->new({
        'container' => $LineItemContainer,
        'payment' => $Payment
    });
    $self->_LogPayment(
        'Mollie order shipment and capture generation request ',
        $OrdersShipmentsHandler->getPayload('POST')
    );

    return $self->_createOrdersShipmentsService($OrdersShipmentsHandler);
}

#----------------------------------------------------------------------------------------
sub _createOrdersShipmentsService {
    my $self = shift;
    my ($HandlerObject) = @_;

    return DE_EPAGES::MolliePayments::API::MolliePaymentsService->new($HandlerObject);
}

#----------------------------------------------------------------------------------------
sub _writeContentOutput {
    my $self = shift;
    my ($hData) = @_;

    $self->writeContentOutput($hData);
}

#----------------------------------------------------------------------------------------
sub _LogPayment {
    my $self = shift;
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, $Name, $DebugData, $Section, $Filter);
}

#----------------------------------------------------------------------------------------
sub _formatErrorResponse {
    my $self = shift;
    my ($ErrorName, $Error, $hResponse) = @_;

    return {
        'Error' => {
            'Name' => $ErrorName,
            'Reason' => $Error->message()
        },
        'Response' => $hResponse
    };
}

1;
