#========================================================================================
# package      DE_EPAGES::MolliePayments::UI::PaymentMethodMolliePayments
# base         DE_EPAGES::Payment::UI::PaymentMethod
# state        private
#----------------------------------------------------------------------------------------
# description  UI Controller for payment method
#========================================================================================
package DE_EPAGES::MolliePayments::UI::PaymentMethodMolliePayments;
use base DE_EPAGES::ExternalPayment::UI::PaymentMethodExternalPayment;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath Transaction );
use DE_EPAGES::Payment::API::Table::ShippingPaymentMap qw ( InsertShippingPaymentMap );
use DE_EPAGES::Payment::API::Table::ShopPaymentLogo qw ( ExistsShopPaymentLogo InsertShopPaymentLogo );
use DE_EPAGES::Payment::API::Shop qw ( GetShopPaymentLogos );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );

use DE_EPAGES::MolliePayments::API::Constants qw (
    @SPLIT_GATEWAY_METHODS
    $SPLIT_GATEWAY_METHODS_MAXROWS
    PAYMENT_TYPE_MOLLIE_PAYMENTS
    PROFILES_LIMIT_MOLLIE_PAYMENTS
);
use DE_EPAGES::MolliePayments::API::MolliePaymentsService;
use DE_EPAGES::MolliePayments::API::MolliePaymentsUrlBuilder;
use DE_EPAGES::MolliePayments::API::Requests::Tokens;
use DE_EPAGES::MolliePayments::API::Requests::Profiles;
use DE_EPAGES::MolliePayments::API::Tools qw ( FilterTokensResponse FilterOnboardingReturnQuery );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use JSON;
use DE_EPAGES::MolliePayments::API::Requests::Methods;

use strict;

#========================================================================================
# function     ViewSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller to view settings on the MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewSettings {
    my $self = shift;
    my ($Servlet) = @_;

    $self->_getTokensOnProvidedCode($Servlet);

    my $PaymentMethod = $Servlet->object();

    my $hCredentials = $PaymentMethod->get([qw(AccessToken RefreshToken)]);
    my $isTokenSet = isValid($hCredentials->{'AccessToken'}) && isValid($hCredentials->{'RefreshToken'});
    $Servlet->vars('IsTokenSet', $isTokenSet);

    my $UrlBuilder = $self->_createMolliePaymentsUrlBuilder();
    my $CurrentUrl = $self->_getCurrentURL($Servlet, $PaymentMethod);
    $Servlet->vars('ConnectAndReconnectUrl', $UrlBuilder->buildConnectAndReconnectUrl($CurrentUrl));

    my $hProfilesList = {};

    if($isTokenSet)
    {
        $hProfilesList = $self->_getProfileList($Servlet, $PaymentMethod);
    }

    $Servlet->vars( 'ProfilesList', $hProfilesList);

    my $hSavedSettings = $PaymentMethod->get([qw(ProfileId IsInTestMode Method)]);

    my $hPaymentMethods;
    if ($isTokenSet && isValid($hSavedSettings->{'ProfileId'}) && isValid($hSavedSettings->{'IsInTestMode'})) {
        $hPaymentMethods = $self->_getPaymentMethodsList($Servlet, $PaymentMethod, $hSavedSettings);
        $Servlet->vars( 'MethodsList', $hPaymentMethods );
    }

    if ($hPaymentMethods && $hPaymentMethods->{'methods'}) {
        my @EnabledMethodsId = map { $_->{'id'} } @{$hPaymentMethods->{'methods'}};
        $self->ViewSettingsMultiPayment($Servlet, 'PaymentMethodMolliePayments', 'Method', \@EnabledMethodsId);
    }

    return;
}

#========================================================================================
# function     SaveSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       SaveSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller to save settings on the MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    my $Form = $Servlet->form();
    my $Shop = $PaymentMethod->getSite();
    my $Folder = $Shop->folder('PaymentMethods');

    # In order to show the sub payment method in the general page of the payment methods
    # we rewrite the Name and Alias before we save the payment method
    my $SplitGatewayMethod = $Form->formValues()->{'Method'}[0];

    if (isValid($SplitGatewayMethod)) {
        my $aSplityGatewayMethod = $self->GetSplitGatewayMethods();
        my %hSPLIT_GATEWAY_METHODS = map { ($_->{'ID'}) => $_ } @$aSplityGatewayMethod;
        my $aliasMethod = $hSPLIT_GATEWAY_METHODS{$SplitGatewayMethod}->{'Name'};
        my $LogoName = $hSPLIT_GATEWAY_METHODS{$SplitGatewayMethod}->{'LogoName'};

        if (not isValid($aliasMethod)) {
            $Form->addFormError({
                'Form'  => 'PaymentMethodMolliePayments',
                'Reason'=> 'OBJECT_EXISTS',
                'Name'  => $SplitGatewayMethod
            });
            $Form->executeFormError;
        }

        #Check if the alias for payment method already exists, before rewriting
        if ($PaymentMethod->alias() ne $aliasMethod) {
            $aliasMethod = $self->_getUniqueAliasForPayment($Folder, $aliasMethod)
        }

        # Rewrite the name and the alias of the payment method
        $PaymentMethod->set({
            'Name' => $aliasMethod,
            'Alias' => $aliasMethod
        });

        # Set Method Logo
        my $hLogoData = $self->_getPaymentLogoData($Shop, $LogoName);
        if (isValid($hLogoData->{'PaymentLogoPath'})) {
            $PaymentMethod->set({ 'InternalImagePath' => $hLogoData->{'PaymentLogoPath'} });
        }
    }

    $Form->form($PaymentMethod, 'MolliePaymentsSettings');
    $self->Save($Servlet);

    return;
}


#========================================================================================
# function     GetSplitGatewayMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSplitGatewayMethods = GetSplitGatewayMethods( );
#----------------------------------------------------------------------------------------
# description  It provides the multi-payment structure of grouped payment methods.
#----------------------------------------------------------------------------------------
# return       $aSplitGatewayMethods | hash with split gateway methods | ref.array
#========================================================================================
sub GetSplitGatewayMethods {
    my $self = shift;

    return \@SPLIT_GATEWAY_METHODS;
}


#========================================================================================
# function     GetSplitGatewayMethodsMaxRow
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MaxRow = GetSplitGatewayMethodsMaxRow( );
#----------------------------------------------------------------------------------------
# description  It provides the maximum number of payment methods of any group.
#----------------------------------------------------------------------------------------
# return       $MaxRow | maximum number of payment methods per group| integer
#========================================================================================
sub GetSplitGatewayMethodsMaxRow {
    my $self = shift;

    return $SPLIT_GATEWAY_METHODS_MAXROWS;
}

#========================================================================================
# function     GetSplitGatewayMethodsDialogOneColumn
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DialogOneColumn = $self->GetSplitGatewayMethodsDialogOneColumn( );
#----------------------------------------------------------------------------------------
# description  Provides the dialog columns formatting. When true the dialog is shown
#               in one column, when false on four columns
#----------------------------------------------------------------------------------------
# return       $DialogOneColumn | If true dialog one column, false four columns | boolean
#========================================================================================
sub GetSplitGatewayMethodsDialogOneColumn {
    my $self = shift;

    return 1;
}

#========================================================================================
# function     NewPaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       NewPaymentMethod( $Servlet, $PaymentMethod, $SplitGatewayMethod, $MethodName, $LogoName );
#----------------------------------------------------------------------------------------
# description  It creates a new payment method.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
# input        $PaymentMethod | payment method object | object
# input        $SplitGatewayMethod | payment method object | object
# input        $MethodName | method name | string
# input        $LogoName | logo name | string
# return       $NewPaymentMethod | payment method | object
#========================================================================================
sub NewPaymentMethod {
    my $self = shift;
    my ( $Servlet, $PaymentMethod, $SplitGatewayMethod, $MethodName, $LogoName ) = @_;

    my $Shop = $PaymentMethod->getSite();
    my $PaymentType = LoadObjectByPath('/PaymentTypes/MolliePayments');

    return unless defined $SplitGatewayMethod && $SplitGatewayMethod =~ /\S/;

    my $hLogoData = $self->_getPaymentLogoData($Shop, $LogoName);

    my $NewPaymentMethod;
    eval {
        Transaction( sub {
            my $hCommonCredentials = $PaymentMethod->get([ qw(AccessToken RefreshToken ProfileId ) ]);
            $NewPaymentMethod = $PaymentType->newPaymentMethod($Shop, {
                'AccessToken' => $hCommonCredentials->{'AccessToken'},
                'RefreshToken' => $hCommonCredentials->{'RefreshToken'},
                'ProfileId' => $hCommonCredentials->{'ProfileId'},
                'Method' => $SplitGatewayMethod,
                'Alias' => $MethodName,
                'Position' => 9999,
                'Shop' => $Shop,
            });

            # activate on all shipping methods
            InsertShippingPaymentMap({
                'ShippingMethodID'  => $_->id,
                'PaymentMethodID'   => $NewPaymentMethod->id()
            }) foreach @{$Shop->get('ShippingMethods')};

            # insert new Logos
            if ($hLogoData->{'PaymentTypeLogoID'}) {
                InsertShopPaymentLogo({
                    'ShopID'            => $Shop->id,
                    'PaymentTypeLogoID' => $hLogoData->{'PaymentTypeLogoID'},
                }) unless ExistsShopPaymentLogo({
                    'ShopID'            => $Shop->id,
                    'PaymentTypeLogoID' => $hLogoData->{'PaymentTypeLogoID'},
                });
            }

            if ($hLogoData->{'PaymentLogoPath'}) {
                $NewPaymentMethod->set({ 'InternalImagePath' => $hLogoData->{'PaymentLogoPath'} });
            }
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'FeatureLimitExceededGeneral') {
            $Servlet->vars('FeatureMaxValueReachedGeneral',1);
        }
        elsif ($Error->code  eq 'FeatureLimitExceeded'){
            $Servlet->vars('FeatureMaxValueReached', $Error->vars);
        }
        else {
            Error($Error);
        }
    }

    return $NewPaymentMethod;
}

#========================================================================================
# function     NewMultiPaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       NewMultiPaymentMethod( $Servlet );
#----------------------------------------------------------------------------------------
# description  Payment method replication with a different method.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub NewMultiPaymentMethod {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    my $PaymentMethod = $Servlet->object();
    my $Values = $Servlet->form->formValues();

    my $aSplitGatewayMethods = $self->GetSplitGatewayMethods();

    # Go through the payment methods array and generate a hash with key = ID and value the payment method.
    my %hSplitGatewayMethods = map { ($_->{'ID'}) => $_ } @$aSplitGatewayMethods;

    # We build an array with the selected methods in the settings page
    my @aSelectedSplitGatewayMethods = map { /^aMultiPaymentMethod_(.*)/x } grep { /^aMultiPaymentMethod_/x } keys %$Values;

    # We check the selected methods
    my $LeftPaymentMethods = $self->_checkNewPaymentMethods($Servlet, $PaymentMethod, \@aSelectedSplitGatewayMethods, \%hSplitGatewayMethods);

    if ( $LeftPaymentMethods > 0 ) {
        splice(@aSelectedSplitGatewayMethods, -1 * $LeftPaymentMethods);
        $Form->addFormError({
                'Form'  => 'PaymentMethod',
                'Reason'=> 'FEATURE_LIMIT_EXCEEDED',
                'Name'  => 'FEATURE_LIMIT_EXCEEDED'
            });
    }

    # We create the diferent selected payment method and add them to an array
    my @newPaymentMethods;
    my $Shop = $PaymentMethod->getSite();
    my $Folder = $Shop->folder('PaymentMethods');
    my $aliasMethod;

    foreach my $SplitGatewayMethod (@aSelectedSplitGatewayMethods) {
        #Check if the alias for payment method already exists, before rewriting
        $aliasMethod = ($hSplitGatewayMethods{$SplitGatewayMethod}->{'UIName'} // $hSplitGatewayMethods{$SplitGatewayMethod}->{'Name'});

        push(@newPaymentMethods,
             $self->NewPaymentMethod(
                $Servlet,
                $PaymentMethod,
                $SplitGatewayMethod,
                $self->_getUniqueAliasForPayment($Folder, $aliasMethod),
                $hSplitGatewayMethods{$SplitGatewayMethod}->{'LogoName'}
             )
        );
    }

    $PaymentMethod->getSite()->child('PaymentMethods')->renumberChildren;
    $Servlet->vars('NewPaymentMethods',\@newPaymentMethods);
    $Form->executeFormError();

    return;
}
#==============================================================================
sub _getUniqueAliasForPayment{
    my $self = shift;
    my ($Folder, $aliasMethod) = @_;

    if ($Folder->existsChild($aliasMethod)) {
        my $i = 1;
        my $newAliasMethod = $aliasMethod . '_' .  $i;
        while ($Folder->existsChild($newAliasMethod)) {
            $newAliasMethod =  $aliasMethod . '_' .  ++$i;
        }
        $aliasMethod = $newAliasMethod;
    }
    return $aliasMethod;
}
#==============================================================================
sub _checkNewPaymentMethods{
    my $self = shift;
    my ($Servlet, $PaymentMethod, $aSelectedSplitGatewayMethods, $hSplitGatewyMethods) = @_;

    my $Shop = $PaymentMethod->getSite();
    my $Form = $Servlet->form();
    my $ExecuteError = 0;

    if (not scalar @$aSelectedSplitGatewayMethods) {
        $Form->addFormError({
            'Form'  => 'PaymentMethod',
            'Reason'=> 'NOTHING_SELECTED',
            'Name'  => 'NOTHING_SELECTED'
        });
    }

    my $LeftPaymentMethods = $Shop->featureCurrentValue('PaymentMethods') + scalar(@$aSelectedSplitGatewayMethods) - $Shop->featureMaxValue('PaymentMethods');
    if ( $LeftPaymentMethods >= scalar(@$aSelectedSplitGatewayMethods) ){
        $ExecuteError = 1;
        $Form->addFormError({
                'Form'  => 'PaymentMethod',
                'Reason'=> 'FEATURE_LIMIT_EXCEEDED',
                'Name'  => 'FEATURE_LIMIT_EXCEEDED'
            });
    }
    if ( $ExecuteError == 1 ){
        $Form->executeFormError();
    }

    return $LeftPaymentMethods;
}

#----------------------------------------------------------------------------------------
sub _getPaymentMethodsList {
    my $self = shift;
    my ($Servlet, $PaymentMethod, $hSavedSettings) = @_;

    my $IsInTestMode = $hSavedSettings->{'IsInTestMode'};
    my $ProfileId = $hSavedSettings->{'ProfileId'};

    my $hMethodsData = {
        'payment' => $PaymentMethod
    };
    my $MethodsHandler = $self->_createMethodsHandler($hMethodsData);
    $MethodsHandler->withProfileId($ProfileId);
    my $MollieService = $self->_createMolliePaymentsService($MethodsHandler);

    my $hLogParams = {
        'PaymentMethodId' => $PaymentMethod->id(),
        'profileId' => $ProfileId,
    };
    my $hMethodsResponse = {};

    eval {
        $hMethodsResponse = $MollieService->withTestMode($IsInTestMode)->doGetRequest();
    };
    if (ExistsError()) {
        my $Error = GetError();
        $hLogParams->{'Error'} = $Error->message();
        $self->_LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS,
                'Error when getting Payment Methods', $hLogParams, undef, undef
        );
        $Servlet->vars('MollieConnectionError', 1);
        return {};
    }

    $hLogParams->{'MethodsResponse'} = $hMethodsResponse;
    $self->_LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, 'Get Payment Methods response', $hLogParams, undef , undef);

    return $hMethodsResponse;
}
#----------------------------------------------------------------------------------------
sub _createMethodsHandler {
    my $self = shift;
    my ($hMethodsData) = @_;

    return DE_EPAGES::MolliePayments::API::Requests::Methods->new($hMethodsData);
}
#----------------------------------------------------------------------------------------
sub _createMolliePaymentsService {
    my $self = shift;
    my ($HandlerObject) = @_;

    return DE_EPAGES::MolliePayments::API::MolliePaymentsService->new($HandlerObject);
}
#----------------------------------------------------------------------------------------
sub _getCurrentURL {
    my $self = shift;
    my ( $Servlet, $PaymentMethod ) = @_;

    my $hFormValues = $Servlet->params();
    my $hParams = {
        'ViewAction' => $hFormValues->{'ViewAction'},
        'ObjectID'   => $PaymentMethod->id(),
    };
    my $hOptions = {
        'Type' => 'admin',
        'UseSSL' => 1,
    };

    return BuildShopUrl($PaymentMethod, $hParams, $hOptions, $Servlet);
}

sub _LogPayment {
    my $self = shift;
    my ($PaymentTypeAlias, $Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment($PaymentTypeAlias, $Name, $DebugData, $Section, $Filter);
}

sub _createMolliePaymentsUrlBuilder {
    my $self = shift;
    return DE_EPAGES::MolliePayments::API::MolliePaymentsUrlBuilder->new();
}

sub _getTokensOnProvidedCode {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form();
    my $AuthorizationCode = $Form->queryValue('code');

    return unless (defined $AuthorizationCode);

    my $PaymentMethod = $Servlet->object();
    my $Service = $self->_buildTokenService({
        'payment' => $PaymentMethod,
        'code' => $AuthorizationCode
    });

    eval {
        my $Response = $Service->doPostRequest();
        $PaymentMethod->set({
            'AccessToken' => $Response->{'access_token'},
            'RefreshToken' => $Response->{'refresh_token'}
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        $self->_LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS,
                'Error when getting tokens', $Error->message(), undef, \&FilterTokensResponse
        );
        $Servlet->vars('MollieConnectionError', 1);
    }

    $self->_LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS,
        'Redirected from changeAction BacklinkMolliePaymentsOnboarding. Return data from Mollie',
        $Form->queryValues(),undef , \&FilterOnboardingReturnQuery
    );
}

sub _buildTokenService {
    my $self = shift;
    my ($hTokenData) = @_;

    my $TokenRequestHelper = DE_EPAGES::MolliePayments::API::Requests::Tokens->new($hTokenData);
    $TokenRequestHelper->withAuthorizationCode();

    return DE_EPAGES::MolliePayments::API::MolliePaymentsService->new($TokenRequestHelper);
}

sub _getProfileList {
    my $self = shift;
    my ($Servlet, $PaymentMethod) = @_;

    TestObject(
        'PaymentMethodMolliePayments',
        $PaymentMethod,
        'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments'
    );

    my $ProfileRequest = $self->_createProfilesRequest($PaymentMethod);
    my $ProfileService = $self->_createMolliePaymentsService($ProfileRequest);

    my $hProfilesList = {};
    eval {
        $hProfilesList = $ProfileService->doGetRequest();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);

        $self->_LogPayment(
            PAYMENT_TYPE_MOLLIE_PAYMENTS,
            'Error when getting profiles',
            $Error->message(),
            undef,
            \&FilterTokensResponse
        );
        $Servlet->vars('MollieConnectionError', 1);
    }

    return $hProfilesList;
}

sub _createProfilesRequest {
    my $self = shift;
    my ($PaymentMethod) = @_;

    my $hData = {
        'payment' => $PaymentMethod
    };

    my $ProfileRequest = DE_EPAGES::MolliePayments::API::Requests::Profiles->new($hData);

    my $hInfo = $PaymentMethod->get([qw( ProfileId )]);
    if (isValid($hInfo->{'ProfileId'})) {
        $ProfileRequest->withFrom($hInfo->{'ProfileId'});
    }

    $ProfileRequest->withLimit(PROFILES_LIMIT_MOLLIE_PAYMENTS);

    return $ProfileRequest;

}

sub _getPaymentLogoData {
    my $self = shift;
    my ($Shop, $LogoName) = @_;

    my $aLogos = GetShopPaymentLogos($Shop, LoadObjectByPath('/PaymentTypes/MolliePayments'));
    my $hLogoData = {};
    foreach my $hLogo (@$aLogos) {
        if ($LogoName eq $hLogo->{'PaymentLogo'}->alias()) {
            $hLogoData->{'PaymentTypeLogoID'} = $hLogo->{'PaymentTypeLogo'}->id;
            $hLogoData->{'PaymentLogoPath'} = $hLogo->{'PaymentLogo'}->get('Image');
        }
    }

    return $hLogoData;
}

1;