#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::Hooks_OrderProcess
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::MolliePayments::Hooks::OrderProcess
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::UI_BasketOrder;
use base DE_EPAGES::Core::API::Test;

use Test::Exception;
use Test::More;
use Test::Mockify;
use Test::Mockify::Sut;
use Test::Mockify::Matcher qw ( Object HashRef String );
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );

use strict;

use constant BASKET_ID => 20803;

use DE_EPAGES::MolliePayments::API::Constants qw (
    PAYMENT_STATUS_PAID
    PAYMENT_STATUS_FAILED
    PAYMENT_STATUS_PENDING
);

sub test {
    my $self = shift;

    $self->_test_AfterSaveMultiCheckoutConfirmation_OrderIsPaid();
    $self->_test_AfterSaveMultiCheckoutConfirmation_OrderIsNotPaid();
    $self->_test_AfterSaveMultiCheckoutConfirmation_OrderIsNotPaid_Pending();
    $self->_test_AfterSaveMultiCheckoutConfirmation_OrderIsNotPaid_Empty();

    done_testing();
}

sub _test_AfterSaveMultiCheckoutConfirmation_OrderIsPaid {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    note('---- OrderIsPaid ---- ');

    my $MockedLineItemPayment = $self->_createLineItemPayment({
        'TransStatus' => PAYMENT_STATUS_PAID,
    });

    my $MockedServlet = $self->_createServlet();
    my $SUT = $self->_createSUT(PAYMENT_STATUS_PAID);

    lives_ok(sub {
        $SUT->ValidateOrderPaymentStatus($MockedServlet, $MockedLineItemPayment)},
        "$SubTestName - ValidateOrderPaymentStatus called lives ok.");

    ok(! WasCalled(
        $MockedServlet->form(), 'executeFormError'),
        "$SubTestName - executeFormError in ValidateOrderPaymentStatus was not called."
    );

}


sub _test_AfterSaveMultiCheckoutConfirmation_OrderIsNotPaid {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    note('---- OrderIsNotPaid ---- ');

    my $MockedLineItemPayment = $self->_createLineItemPayment({
        'TransStatus' => PAYMENT_STATUS_FAILED,
    });

    my $MockedServlet = $self->_createServlet();
    my $SUT = $self->_createSUT(PAYMENT_STATUS_FAILED);

    $SUT->ValidateOrderPaymentStatus($MockedServlet, $MockedLineItemPayment);

    ok(WasCalled(
        $MockedServlet->form(), 'executeFormError'),
        "$SubTestName - executeFormError in ValidateOrderPaymentStatus was called."
    );

    is_deeply(
        GetParametersFromMockifyCall($MockedServlet->form(), 'executeFormError')->[0],
        {
            'Name' => 'MolliePaymentsFailed',
            'Reason' => 'card_declined'
        },
        "$SubTestName - Mollie orders was initiated with correct parameters"
    );
}

sub _test_AfterSaveMultiCheckoutConfirmation_OrderIsNotPaid_Pending {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    note('---- OrderIsNotPaid Pending ---- ');

    my $MockedLineItemPayment = $self->_createLineItemPayment({
        'TransStatus' => PAYMENT_STATUS_PENDING,
    });

    my $MockedServlet = $self->_createServlet();
    my $SUT = $self->_createSUT(PAYMENT_STATUS_PENDING);

    $SUT->ValidateOrderPaymentStatus($MockedServlet, $MockedLineItemPayment);

    ok(! WasCalled(
        $MockedServlet->form(), 'executeFormError'),
        "$SubTestName - executeFormError in ValidateOrderPaymentStatus was not called."
    );

}

sub _test_AfterSaveMultiCheckoutConfirmation_OrderIsNotPaid_Empty {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    note('---- OrderIsNotPaid Empty ---- ');

    my $MockedLineItemPayment = $self->_createLineItemPayment({
        'TransStatus' => '',
    });

    my $MockedServlet = $self->_createServlet();
    my $SUT = $self->_createSUT('');

    $SUT->ValidateOrderPaymentStatus($MockedServlet, $MockedLineItemPayment);

    ok(! WasCalled(
        $MockedServlet->form(), 'executeFormError'),
        "$SubTestName - executeFormError in ValidateOrderPaymentStatus was not called."
    );

}


sub _createSUT {
    my $self = shift;
    my ($MockedMolliePaymentStatus) = @_;

    my $MockifySut = Test::Mockify::Sut->new(
        'DE_EPAGES::MolliePayments::UI::BasketOrder');

    $MockifySut->mock('_createPaymentStatusService')
        ->when()->thenReturn($self->_createPaymentStatusService($MockedMolliePaymentStatus));

    $MockifySut->mock('_LogPayment')
        ->when(String(), HashRef() )->thenReturnUndef();

    return $MockifySut->getMockObject();
}

sub _createPaymentStatusService {
    my $self = shift;
    my ($MockedMolliePaymentStatus) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::PaymentStatusService');
    $Mockify->mock('getMollieOrderAndHandleStatus')
        ->when(Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'))
        ->thenReturn($MockedMolliePaymentStatus);
    $Mockify->mock('deleteLineItemPaymentStatusInfo')
        ->when(Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'))
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createServlet {
    my $self = shift;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($self->_createBasket());
    $MockifyServlet->addMockWithReturnValue( 'form', $self->_createForm() );

    return $MockifyServlet->getMockObject();
}

sub _createForm {
    my $self = shift;

    my $Mockify = Test::Mockify->new( 'DE_EPAGES::Presentation::API::Form');
    $Mockify->mock('executeFormError')->when(HashRef())->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createBasket {
    my $self = shift;
    my ($hAttributes) = @_;

    $hAttributes->{'ID'} = BASKET_ID;
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::Basket',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

sub _createLineItemPayment {
    my $self = shift;
    my ($hAttributes) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments',
        $hAttributes
    );

    $Mockify->addMockWithReturnValueAndParameterCheck('instanceOf', 1, ['string']);

    return $Mockify->getMockObject();
}


#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
