#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::UI_LineItemPaymentMolliePayments
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI::Provider
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::UI_LineItemPaymentMolliePayments;
use base DE_EPAGES::Core::API::Test;

use Test::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;
use Test::Mockify::ReturnValue;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );
use Test::Mockify::Matcher qw ( Object String Number HashRef );

use strict;

sub test {
    my $self = shift;

    $self->test_MolliePaymentsRefunds_Success();
    $self->test_MolliePaymentsRefunds_RefundAmountError();
    $self->test_MolliePaymentsRefunds_DoRefundError();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub test_MolliePaymentsRefunds_Success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hSuccessResponse = {
        'AmountRemaining' => 233,
        'AmountRefunded' => 223,
        {
            'CurrencyID' => 'EUR',
        }
    };
    my $RefundsService = $self->_createRefundsService($hSuccessResponse);
    my $SUT = $self->_createSUT($RefundsService);

    my $hLineItemPaymentAttributes = {
        'ID' => 'IUYIUY-76876876-KJHKJH',
        'AmountRemaining' => 1,
        'AmountRefunded' => undef,
        'CurrencyID' => 'EUR',
    };

    my $LineItemPayment = $self->_createLineItemPayment($hLineItemPaymentAttributes);
    my $hFormAttributes = {
        'refundAmount' => 3.5,
        'throwError' => 0
    };
    my $Form = $self->_createForm($hFormAttributes);
    my $Servlet = $self->_createServlet($LineItemPayment, $Form);

    $SUT->MolliePaymentsRefunds($Servlet);

    is(  WasCalled( $Form, 'mapFormError' ), 0,
                "$SubTestName - mapFormError was not called in Form."
    );

    is(  WasCalled( $Form, 'executeFormError' ), 0,
                "$SubTestName - executeFormError was not called in Form."
    );

    is_deeply( $Servlet->vars('ResponseContent'), $hSuccessResponse,
                "$SubTestName - Servlet ResponseContent was correctly set."
    );

    return;
}

sub test_MolliePaymentsRefunds_RefundAmountError {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hSuccessResponse = {
        'AmountRemaining' => 233,
        'AmountRefunded' => 223,
        {
            'CurrencyID' => 'EUR',
        }
    };
    my $RefundsService = $self->_createRefundsService($hSuccessResponse);
    my $SUT = $self->_createSUT($RefundsService);

    my $hLineItemPaymentAttributes = {
        'ID' => 'IUYIUY-76876876-KJHKJH',
        'AmountRemaining' => 1,
        'AmountRefunded' => undef,
        'CurrencyID' => 'EUR',
    };

    my $LineItemPayment = $self->_createLineItemPayment($hLineItemPaymentAttributes);
    my $hFormAttributes = {
        'refundAmount' => 3.5,
        'throwError' => 'FLOAT_TOO_LARGE'
    };
    
    my $Form = $self->_createForm($hFormAttributes);
    my $Servlet = $self->_createServlet($LineItemPayment, $Form);

    $SUT->MolliePaymentsRefunds($Servlet);

    ok(  WasCalled( $Form, 'mapFormError' ),
                "$SubTestName - mapFormError was called in Form."
    );

    is(  WasCalled( $Form, 'executeFormError' ), 1,
                "$SubTestName - executeFormError was not called in Form."
    );

    is_deeply(  GetParametersFromMockifyCall( $Form, 'executeFormError'), [],
                "$SubTestName - executeFormError was correctly called without arguments."
    );

    is_deeply(  GetParametersFromMockifyCall( $Form, 'mapFormError')->[0],
    {
        'Name'       => 'Amount',
        'Type'       => 'reg_money',
        'Mandatory'  => 1,
        'MinValue'   => 0.01,
        'MaxValue'   => $hLineItemPaymentAttributes->{'AmountRemaining'},
        'CurrencyID' => $hLineItemPaymentAttributes->{'CurrencyID'}
    },
                "$SubTestName - executeFormError was correctly called with arguments."
    );

    return;
}

sub test_MolliePaymentsRefunds_DoRefundError {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hSuccessResponse = {
        'AmountRemaining' => 233,
        'AmountRefunded' => 223,
        {
            'CurrencyID' => 'EUR',
        }
    };
    my $RefundsService = $self->_createRefundsService($hSuccessResponse, undef, 'ERROR_IN_DOREFUND');
    my $SUT = $self->_createSUT($RefundsService);

    my $hLineItemPaymentAttributes = {
        'ID' => 'IUYIUY-76876876-KJHKJH',
        'AmountRemaining' => 1,
        'AmountRefunded' => undef,
        'CurrencyID' => 'EUR',
    };

    my $LineItemPayment = $self->_createLineItemPayment($hLineItemPaymentAttributes);
    my $hFormAttributes = {
        'refundAmount' => 3.5,
        'throwError' => undef
    };

    my $Form = $self->_createForm($hFormAttributes);
    my $Servlet = $self->_createServlet($LineItemPayment, $Form);

    $SUT->MolliePaymentsRefunds($Servlet);

    is(  WasCalled( $Form, 'mapFormError' ), 0,
                "$SubTestName - mapFormError was not called in Form."
    );

    is(  WasCalled( $Form, 'executeFormError' ), 1,
                "$SubTestName - executeFormError was not called in Form."
    );

    is(  GetParametersFromMockifyCall( $Form, 'executeFormError')->[0]->{'Name'},
        "RefundFailed",
        "$SubTestName - executeFormError was correctly called with arguments."
    );

    return;
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createForm {
    my $self = shift;
    my ($hFormAttributes) = @_;

    my $Mockify = Test::Mockify->new( 'DE_EPAGES::Presentation::API::Form');

    if ($hFormAttributes->{'throwError'}) {
        $Mockify->mock('field')
            ->when(Object(), String(), HashRef())
            ->thenThrowError($hFormAttributes->{'throwError'});
    } else {
        $Mockify->mock('field')
            ->when(Object(), Number(), HashRef())
            ->thenReturn($hFormAttributes->{'refundAmount'});
    }
    $Mockify->mock('mapFormError')->when(HashRef(), Object())->thenReturnUndef();
    $Mockify->mock('executeFormError')->when(HashRef())->thenReturnUndef();
    $Mockify->mock('executeFormError')->when()->thenReturnUndef();
    $Mockify->mock('formValue')->when(String('Amount'))->thenReturn($hFormAttributes->{'refundAmount'});

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;
    my ($Object, $Form) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Object);
    $MockifyServlet->addMockWithReturnValue( 'form', $Form );

    return $MockifyServlet->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($RefundsService) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::UI::LineItemPaymentMolliePayments');

    $Mockify->mock( '_createRefundsService')->when()->thenReturn($RefundsService);
    $Mockify->mock( '_LogPayment')->when(String(), HashRef())->thenReturnUndef();

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createRefundsService {
    my $self = shift;
    my ($hSuccessResponse, $ThrowErrorUpdate, $ThrowErrorDoRefund) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::MolliePaymentsRefundsService');

    if ($ThrowErrorUpdate) {
        $Mockify->mock('updateRefundAmountsByGetMolliePayment')
            ->when(Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'))
            ->thenThrowError($ThrowErrorUpdate);
    } else {
        $Mockify->mock('updateRefundAmountsByGetMolliePayment')
            ->when(Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'))
            ->thenReturnUndef();
    }

    if ($ThrowErrorDoRefund) {
        $Mockify->mock('doRefund')->when(Object(), Number())->thenThrowError($ThrowErrorDoRefund);
    } else {
        $Mockify->mock('doRefund')->when(Object(), Number())->thenReturn($hSuccessResponse);
    }

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createLineItemPayment {
    my $self = shift;
    my ($hAttributes) = @_;

    

    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments', $hAttributes
    );

    return $MockifyContainer->getMockObject();
}

__PACKAGE__->runScript();
