#==============================================================================
# package      DE_EPAGES::Moneybookers::UI::PaymentMethodMoneybookers
# state        private
#==============================================================================
package DE_EPAGES::Moneybookers::UI::PaymentMethodMoneybookers;
use base DE_EPAGES::ExternalPayment::UI::PaymentMethodExternalPayment;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Moneybookers::API::Constants qw (
    $SPLIT_GATEWAY_METHODS_MAXROWS
    @SPLIT_GATEWAY_METHODS
);
use DE_EPAGES::Moneybookers::API::Mail qw ( SendMoneybookersRegistration );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadRootObject Transaction );
use DE_EPAGES::Payment::API::Shop qw ( GetShopPaymentLogos );
use DE_EPAGES::Payment::API::Table::ShippingPaymentMap qw ( InsertShippingPaymentMap );
use DE_EPAGES::Payment::API::Table::ShopPaymentLogo qw (
    ExistsShopPaymentLogo
    InsertShopPaymentLogo
);

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::View($Servlet);

    $self->ViewSettingsMultiPayment($Servlet, 'PaymentMethodMoneybookers', 'SplitGatewayMethod');

    return;
}

sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();

    #--- check form values
    $Servlet->form->form($PaymentMethod, 'MoneybookersSettings');

    #--- standard save
    $self->SUPER::Save($Servlet);

    return;
}


#========================================================================================
# function     NewMultiPaymentMethodMoneybookers
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->NewMultiPaymentMethodMoneybookers( $Servlet );
#----------------------------------------------------------------------------------------
# description  Payment method replication with a different method.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub NewMultiPaymentMethodMoneybookers {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::NewMultiPaymentMethod($Servlet);

    return ;
}


#==============================================================================
# function     NewPaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->NewPaymentMethod( $Servlet, $PaymentMethod, $SplitGatewayMethod, $MethodName, $LogoName );
#----------------------------------------------------------------------------------------
# description  It creates a new payment method.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
# input        $PaymentMethod | payment method object | object
# input        $SplitGatewayMethod | payment method object | object
# input        $MethodName | method name | string
# input        $LogoName | logo name | string
# return       $NewPaymentMethod | payment method | object
#========================================================================================
sub NewPaymentMethod {
    my $self = shift;
    my ( $Servlet, $PaymentMethod, $SplitGatewayMethod, $MethodName, $LogoName ) = @_;

    my $Shop = $PaymentMethod->getSite();
    my $PaymentType = LoadObjectByPath('/PaymentTypes/Moneybookers');

    return unless defined $SplitGatewayMethod && $SplitGatewayMethod =~ /\S/;

    my $aLogos = GetShopPaymentLogos($Shop, LoadObjectByPath('/PaymentTypes/Moneybookers'));
    #my $SplitGatewayMethodAlias;
    my $PaymentTypeLogoID;
    foreach my $hLogo (@$aLogos) {
        if ($LogoName eq $hLogo->{'PaymentLogo'}->alias) {
            $PaymentTypeLogoID = $hLogo->{'PaymentTypeLogo'}->id;
        }
    }

    eval {
        Transaction( sub {
            $PaymentMethod = $PaymentType->newPaymentMethod($Shop, {
                'MerchantID' => $PaymentMethod->get('MerchantID'),
                'Email' => $PaymentMethod->get('Email'),
                'Secret' => $PaymentMethod->get('Secret'),
                'SplitGatewayMethod' => $SplitGatewayMethod,
                'Alias' => $MethodName,
                'Position' => 9999,
                'Shop' => $Shop,
            });

            # activate on all shipping methods
            InsertShippingPaymentMap({
                'ShippingMethodID'  => $_->id,
                'PaymentMethodID'   => $PaymentMethod->id
            }) foreach @{$Shop->get('ShippingMethods')};

            #insert new Logos
            if ($PaymentTypeLogoID){
                InsertShopPaymentLogo({
                    'ShopID'            => $Shop->id,
                    'PaymentTypeLogoID' => $PaymentTypeLogoID,
                }) unless ExistsShopPaymentLogo({
                    'ShopID'            => $Shop->id,
                    'PaymentTypeLogoID' => $PaymentTypeLogoID,
                });
            }
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'FeatureLimitExceededGeneral') {
            $Servlet->vars('FeatureMaxValueReachedGeneral',1);
        }
        elsif ($Error->code  eq 'FeatureLimitExceeded'){
            $Servlet->vars('FeatureMaxValueReached', $Error->vars);
        }
        else {
            Error($Error);
        }
    }


    return $PaymentMethod;
}

#==============================================================================
sub MoneybookersRegistration {
    my $self = shift;
    my ($Servlet) = @_;

    my $System = LoadRootObject();
    my $PaymentMethod = $Servlet->object;
    my $Shop = $PaymentMethod->getSite();
    my $User = $Servlet->user;
    my $MerchantID = $PaymentMethod->get('MerchantID');
    my $MailType = $System->folder('MailTypes')->child('MoneybookersRegistration');
    eval {
        SendMoneybookersRegistration( $MailType, $Shop, $MerchantID, $User );
    };
    if (ExistsError()) {
        my $Error = GetError();
        $Servlet->form->executeFormError({
            'Reason'=> 'SendEmailFailed',
            'Name'  => 'MoneybookersRegistration',
            'Value' => $Error->shortMessage,
        });
    }
    $Servlet->vars('RegistrationSuccessful', 1);
    return;
}


#========================================================================================
# function     GetSplitGatewayMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSplitGatewayMethods = GetSplitGatewayMethods( );
#----------------------------------------------------------------------------------------
# description  It provides the multi-payment structure of grouped payment methods.
#----------------------------------------------------------------------------------------
# return       $aSplitGatewayMethods | hash with split gateway methods | ref.array
#========================================================================================
sub GetSplitGatewayMethods {
    my $self = shift;

    return \@SPLIT_GATEWAY_METHODS;
}


#========================================================================================
# function     GetSplitGatewayMethodsMaxRow
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MaxRow = GetSplitGatewayMethodsMaxRow( );
#----------------------------------------------------------------------------------------
# description  It provides the maximum number of payment methods of any group.
#----------------------------------------------------------------------------------------
# return       $MaxRow | maximum number of payment methods per group| integer
#========================================================================================
sub GetSplitGatewayMethodsMaxRow {
    my $self = shift;

    return $SPLIT_GATEWAY_METHODS_MAXROWS;
}

#========================================================================================
# function     GetSplitGatewayMethodsDialogOneColumn
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DialogOneColumn = $self->GetSplitGatewayMethodsDialogOneColumn( );
#----------------------------------------------------------------------------------------
# description  Provides the dialog columns formatting. When true the dialog is shown
#               in one column, when false on four columns
#----------------------------------------------------------------------------------------
# return       $DialogOneColumn | If true dialog one column, false four columns | boolean
#========================================================================================
sub GetSplitGatewayMethodsDialogOneColumn {
    my $self = shift;

    return 0;
}

1;
