#==============================================================================
# §package      DE_EPAGES::Order::UI::CustomerOrder
# §state        private
#==============================================================================
package DE_EPAGES::Order::UI::CustomerOrder;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Customer::API::Constants qw ( REGISTRATION_FIELDS_MAX_LENGTH );
use DE_EPAGES::Core::API::Error qw ( Error GetError ExistsError );
use DE_EPAGES::Core::API::File qw ( GetFileContent fsplit );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );
use DE_EPAGES::Core::API::PerlTools qw (GetCurrentDateTime ElementOf isValid);
use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestMultipleEMailAddresses );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias ExistsObject Transaction );
use DE_EPAGES::Order::API::Constants qw ( ORDER_STATUS_ATTRIBUTES );
use DE_EPAGES::Order::API::LineItemContainer qw ( SavePayment SaveShipping );
use DE_EPAGES::Order::API::Order qw ( SendOrderConfirmationMail GetEditableOrder CreateOrderWithContainer
                                      SendOrderDocumentAsPDFMail GetLatestOrderInvoice GenerateOrderDocumentPDF );
use DE_EPAGES::Order::API::Shop qw ( GetNextOrderNo );
use DE_EPAGES::Order::API::TextExport qw (ExportOrdersAsText);
use DE_EPAGES::Order::API::XML::CustomerOrderExport;
use DE_EPAGES::Presentation::API::LocaleHandler qw ( AddLanguageToURL );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Customer::API::Table::CustomerUser qw ( GetPKeyCustomerUsersByCustomerID );
use DE_EPAGES::Shop::API::Table::ShopCurrency qw ( ExistsShopCurrency );
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
#==============================================================================
# Event handlers
#==============================================================================

# test if new order number exists
sub attributeValues {
    my $self = shift;
    my ( $Servlet, $Object, $Class ) = @_;
    TestObject( 'Servlet', $Servlet );
    TestObject( 'Object', $Object );
    TestObject( 'Class', $Class );

    my $hhValues = $self->SUPER::attributeValues( $Servlet, $Object, $Class );
    if (exists $hhValues->{'Values'}{'Alias'} and $Object->alias ne $hhValues->{'Values'}{'Alias'}) {
        my $Orders = LoadClassByAlias('CustomerOrder')->packageObject->dal->get('BySiteIDClassIDAlias', [
            $Object->getSite()->id, $Object->class->id, $hhValues->{'Values'}{'Alias'}
        ]);
        $Servlet->form->addFormError({
            'Reason'=> 'OrderNoExists',
            'Name'  => 'Alias',
        }) if scalar @$Orders;
    }
    return $hhValues;
}

# save changes to the Original Order
# delete the copied order after saving
sub Apply {
    my $self = shift;
    my ($Servlet) = @_;
    my $Order = $Servlet->object;
    my $Form = $Servlet->form;

    RunBatchProcess('UpdateOrders', sub {

        TriggerHook('UI_EditOrder2Order', {'Servlet' => $Servlet});

        # save order attributes in general
        $self->SUPER::Save($Servlet);


        # save order status and send the notification mail if the order status has changed
        if (! $Order->get('OriginalOrder')) {
            my @StatusList = ORDER_STATUS_ATTRIBUTES;
            my $StateChangeDate = GetCurrentDateTime();
            my $hOldStatus = $Order->get(\@StatusList);
            my %NewStatus;
            foreach my $Status (@StatusList) {
                if( $Servlet->params('Status_'.$Status) and not defined $hOldStatus->{$Status} ) {
                    $NewStatus{$Status} = $StateChangeDate;
                }
                elsif( $hOldStatus->{$Status} and defined $Servlet->params('Status_'.$Status) and not $Servlet->params('Status_'.$Status) ) {
                    $NewStatus{$Status} = undef;
                }
            }

            eval {
                Transaction( sub {
                    $Order->set( \%NewStatus )
                });
            };
            if( ExistsError() ) {
                my $Error = GetError();
                Error( $Error ) unless $Error->code eq 'SEND_MAIL_FAILED';

                $Servlet->params('Recipient', $Error->vars()->{'Recipient'});
                $Form->executeFormError( {
                    Reason => $Error->code,
                    Form => 'Form',
                    Name => 'Name',
                } );
            }

            if ( scalar keys %NewStatus > 0 ) {
                TriggerHook(
                    'UI_ChangeCustomerOrderStatus',
                    {
                        'Servlet' => $Servlet,
                        'Order' => $Order,
                        'Status' => \%NewStatus
                    }
                );
            }
        }

        # save also status on copy of order if exists
        if ( (! $Order->get('OriginalOrder')) and GetEditableOrder($Order) ){

            my $OrderCopy = GetEditableOrder($Order);
            # set the Object to save
            $Servlet->params('ObjectID',$OrderCopy->id);
            $self->SUPER::Save($Servlet);
            # reset Object
            $Servlet->params('ObjectID',$Order->id);
        }

        if ($Order->get('OriginalOrder')) {
            my $OrigOrder = $Order->get('OriginalOrder');
            my $Orders = $OrigOrder->parent;
            my $OrigOrderGUID = $OrigOrder->guid();

            Transaction(sub {

                TriggerHook('BeforeEditOrder2Order', {'Order' => $Order});

                # delete the original order and set the parent from the copy to Orders
                $OrigOrder->delete;
                $Order->set({'Parent'=> $Orders,'OriginalOrder'=>undef});

                # keep the GUID of the order so that the order is still available on its URL via the REST API (AD-5326).
                $Order->set({'GUID' => $OrigOrderGUID});

                # order change is confiremed.
                $self->_lastUpdate($Order);
            });

            # reset some vars
            $Servlet->params('ObjectID',$Order->id);
            $Servlet->params('UpdateNotApplied',0);
        }

    });

    return;
}

sub ViewMBO {
    my $self = shift;
    my ($Servlet) = @_;
    my $Order = $Servlet->object;

    my $EditableOrder = $Order->get('OriginalOrder') ? undef : GetEditableOrder($Order);
    if ( $EditableOrder ) {
        $Servlet->params('ObjectID', $EditableOrder->id);
    }

    return ;
}

sub UnityViewMBO {
    my $self = shift;
    my ($Servlet) = @_;

    $self->ViewMBO($Servlet);

    # trigger hook to allow the adding of context sensitive apps
    TriggerHook('UI_CustomerOrder_UnityViewMBO', { 'Servlet' => $Servlet });

    return;
}

#save the changes on addresses and lineitems to the copied order
sub Save {
    my $self = shift;
    my ($Servlet) = @_;
    my $Order = $Servlet->object;
    my $LineItemForm = $Servlet->form->form($Order, 'Save');
    my $LineItemContainer = $Order->get('LineItemContainer');

    # set template to edit mode
    $Servlet->params('UpdateNotApplied',1);

    $self->SUPER::Save($Servlet);

    # check address data
    my %Addresses;
    my %AddressIsEmpty;
    foreach my $AddressType (qw (BillingAddress ShippingAddress)) {
        my $hhValues = $self->SUPER::attributeValues( $Servlet, $Order, LoadClassByAlias('Address'), $AddressType.'.' );
        $Addresses{$AddressType} = $hhValues->{'Values'};

        my $hFieldNames = REGISTRATION_FIELDS_MAX_LENGTH();

        foreach my $FieldName (keys %{$hFieldNames}) {
            my $MaxLength = $hFieldNames->{$FieldName};
            my $FieldValue = $Addresses{$AddressType}{$FieldName};

            next unless ( isValid($FieldValue) && isValid($MaxLength) );

            if(length $FieldValue > $MaxLength) {
                $Servlet->form->executeFormError({
                    'Reason'=> 'INVALID_REGISTRATION_INPUT_FIELD_FORMAT',
                    'Reason'=> 'INPUT_FIELD_FIELD_TOO_LONG',
                    'Name'  => $FieldName,
                });
            }
        }

        $AddressIsEmpty{$AddressType} = (scalar grep { defined $hhValues->{'Values'}{$_} } keys %{$hhValues->{'Values'}}) ? 0 : 1;
    }

    $Servlet->form->executeFormError;

    # save the address data
    foreach my $AddressType (qw (BillingAddress ShippingAddress)) {
        my $hAddress = $Addresses{$AddressType};
        my $OrderAddress = $Order->get($AddressType);

        # delete an empty address
        if (defined $OrderAddress){
            if( $AddressIsEmpty{$AddressType} ) {
                Transaction( sub {
                    $Order->set({$AddressType => undef});
                    $OrderAddress->delete();
                });
            }
            else {
                $OrderAddress->set( $hAddress );
            }
        }
        elsif(! $AddressIsEmpty{$AddressType} ) {
            Transaction( sub {
                $OrderAddress = $Order->insertChild('Address', {'Alias'=>$AddressType} );
                $OrderAddress->set( $hAddress );
                $Order->set({$AddressType => $OrderAddress});
            })
        }
    }


    # set the correct TaxAreaDigital on LineItemContainer
    $LineItemContainer->set({'TaxAreaDigital' => $Order->taxAreaDigitalByAddress()}) if $Order->getSite()->get('TaxMatrixDigital');
    $LineItemContainer->recalculate;


    # update LineItems
    my $Prepayment = $LineItemForm->{'PrepaymentTotal'};
    foreach my $LineItem (grep { defined $_->get('OriginalLineItemPrice')} @{$LineItemContainer->get('LineItems')}) {
        $LineItem->set({'Prepayment' => $Prepayment});
        $Prepayment = 0;
    }

    my $Shop = $Order->getSite;
    my $LineItemClass = LoadClassByAlias('LineItem');
    my $Quantities = $LineItemForm->{'LineItems'};
    foreach my $QuantityEntry (@$Quantities) {
        my $LineItem = $Shop->loadObject($QuantityEntry->{'LineItemID'}, $LineItemClass);
        delete $QuantityEntry->{'LineItemID'};
        if( $LineItem->container->id != $LineItemContainer->id ) {
            die sprintf 'invalid LineItem: %d used in Order: %d', $LineItem->id, $Order->id;
        }

        my $Product = $LineItem->get('Product');
        if( defined $Product ) {
            my $PriceQuantity = $Product->getWithParent('PriceQuantity');
            if(defined $PriceQuantity and $PriceQuantity != 0) {
                $QuantityEntry->{'BasePrice'} = $QuantityEntry->{'BasePrice'} / $PriceQuantity;
            }
        }

        $LineItem->set($QuantityEntry);
    }

    #update discounts
    my $ahDiscountLineItems = $LineItemForm->{'DiscountLineItems'};
    foreach my $hDiscountLineItem (@$ahDiscountLineItems){
        my $ObjectID = $hDiscountLineItem->{'DiscountLineItemID'};
        my $LineItem = $Shop->loadObject($ObjectID, $LineItemClass);
        my $DiscountPrice = $hDiscountLineItem->{'DiscountPrice'};
        # set attributes for detecting the changes by editing the order
        if ($DiscountPrice != $LineItem->get('LineItemPrice')){
            $LineItemContainer->set({'GlobalValueDiscountChanged' => 1 }) ;
            $LineItem->set({'HasManualPrice' => 1 }) if $LineItem->class->existsAttribute('HasManualPrice') ;
        }
        $LineItem->set({'LineItemPrice'=>$DiscountPrice});
    }

    my $Form = $Servlet->form;

    # update tax area
    my $TaxArea = $LineItemContainer->get('TaxArea');
    my $TaxAreaID = $Form->value('TaxArea');
    my $TaxModel = $Form->value('TaxModel');
    if (defined $TaxAreaID and (not defined $TaxArea or $TaxArea->id != $TaxAreaID )) {
        $LineItemContainer->set({
            'TaxArea' => $Shop->loadObject($TaxAreaID, 'TaxArea', { 'AllowSystemSite' => 1 })
        });
    }

    # set the taxmodel given by form parameter
    $LineItemContainer->set({'TaxModel' => $TaxModel }) if defined $TaxModel;

    # set shipping and payment
    my $aWarnings = BlockWarnings(sub{
        my $ShippingMethodID = $Form->value('OrderShippingMethod');
        SaveShipping($LineItemContainer, $Shop->loadObject($ShippingMethodID, 'ShippingMethod')) if $ShippingMethodID;

        my $PaymentMethodID = $Form->value('OrderPaymentMethod');
        SavePayment($LineItemContainer, $Shop->loadObject($PaymentMethodID, 'PaymentMethod')) if $PaymentMethodID;

    }, [qw (ShippingMethodChanged PaymentMethodChanged NoShippingMethod NoPaymentMethod)]);
    $Servlet->form->mapFormError({
        'Form' =>'Save',
        'Name' => 'Order'.$_->vars->{'Name'},
        },$_) foreach @$aWarnings;

    TriggerHook('UI_SaveOrder2Order', {'Servlet' => $Servlet});

    $LineItemContainer->recalculate;
    $Servlet->form->executeFormError();
    return;
}

sub CancelEdit {
    my $self = shift;
    my ($Servlet) = @_;
    my $Order = $Servlet->object;

    my $OriginalOrder = $Order->get('OriginalOrder');

    # delete the Copy of the Order
    $Order->delete if defined $OriginalOrder;

    return;
}

sub EditOrder {
    my $self = shift;
    my ($Servlet) = @_;
    my $Order = $Servlet->object;

    $self->CreateEditableOrder($Servlet, $Order);

    return ;
}

sub SelectAddress {
    my $self = shift;
    my ($Servlet) = @_;
    my $Basket = $Servlet->object;
    my $Shop = $Basket->getSite;
    my $FormValues = $Servlet->form->form($Basket,'SelectAddress');

    my %Info;
    # set billing address
    my $BillingAddress = $FormValues->{'BillingAddress'} == -1
         ? undef
         : $Shop->loadObject($FormValues->{'BillingAddress'}, 'Address');

    # don't allow to view the address of someone else
    $Servlet->testPermission($BillingAddress, 'Save') if $BillingAddress;

    $Info{'BillingAddress'} = $BillingAddress;
    $Info{'IsAddressOK'}    = 1 if defined $BillingAddress;

    # set shipping address
    my $ShippingAddress = ($FormValues->{'ShippingAddress'} == -1 or $FormValues->{'ShippingAddress'} == $FormValues->{'BillingAddress'})
         ? undef
         : $Shop->loadObject($FormValues->{'ShippingAddress'}, 'Address');

    # don't allow to view the address of someone else
    $Servlet->testPermission($ShippingAddress, 'Save') if $ShippingAddress;

    $Info{'ShippingAddress'} = $ShippingAddress;

    # set shop address
    my $ShopAddress = $Basket->get('ShopAddress');
    if (not defined $ShopAddress) {
        $ShopAddress = $Basket->getSite()->get('Address');
        $Info{'ShopAddress'} = $ShopAddress if defined $ShopAddress;
    }
    $Basket->set(\%Info);
    return;
}

sub SetOrderStatus {
    my $self = shift;
    my ($Servlet) = @_;

    my $MasterObject = $Servlet->object;
    my $Shop = $MasterObject->getSite;
    my $Form = $Servlet->form;
    my @States = $Form->value('ChangeAction');
    my $Status = $States[0];

    my $StateChangeDate = GetCurrentDateTime();
    my $hValues = $Form->form($MasterObject, 'ListedObjects');

    my @ListObjects;
    foreach my $hListObjectID (@{$hValues->{'ListObjectIDs'}}){
        next unless ExistsObject($hListObjectID->{'ListObjectID'});
        my $ListObject = $Shop->loadObject( $hListObjectID->{'ListObjectID'}, 'CustomerOrder' );
        $Servlet->testPermission($ListObject, 'Save');
        push @ListObjects,$ListObject;
    }

    foreach my $Order (@ListObjects){
        next if $Order->get($Status);
        # test if an editable order exists and set also the status
        my $EditableOrder = $Order->get('OriginalOrder') ? undef : GetEditableOrder($Order);

        _IgnoreErrorCode('SEND_MAIL_FAILED', sub {
            $EditableOrder->set({ $Status => $StateChangeDate });
        }) if $EditableOrder;

        _IgnoreErrorCode('SEND_MAIL_FAILED', sub {
            Transaction( sub {
                $Order->set({ $Status => $StateChangeDate });
            });
        });

        TriggerHook(
            'UI_ChangeCustomerOrderStatus',
            {
                'Servlet' => $Servlet,
                'Order' => $Order,
                'Status' => { $Status => $StateChangeDate }
            }
        );
    }
    return;
}

# ignore a specific error code in $Sub->(), otherwise re-throw
sub _IgnoreErrorCode {
    my ($ErrorCode, $Sub) = @_;

    eval { $Sub->() };
    return unless ExistsError();

    my $Error = GetError();
    return if $Error->code eq $ErrorCode;

    Error($Error);
}

sub GenerateDocuments {
    my $self = shift;
    my ($Servlet) = @_;
    return;
}

sub SendConfirmationMail {
    my $self = shift;
    my ($Servlet) = @_;
    my $Form = $Servlet->form;
    my $Order = $Servlet->object;

    my $BillingAddress = $Order->get('BillingAddress') // $Order->get('Customer')->get('BillingAddress');
    my $Email = $BillingAddress->get('EMail');
    if (defined $Email) {
        my $aWarnings = eval {BlockWarnings( sub {SendOrderConfirmationMail($Order); }, ['No_Recipients','Invalid_Recipient','InactiveMailType','MISSING_FROM']);};

        if (ExistsError()) {
            my $Error = GetError();
            LogError($Error);
            $Form->executeFormError( {
                Reason => 'SEND_MAIL_FAILED',
                Form => 'Form',
                Name => 'Name',
            } );
        }

        $Servlet->form->mapFormError({'Name'=>'EMail'}, $_) foreach @$aWarnings;
        $Servlet->form->executeFormError;
        $Servlet->vars('ORDERCONF_SEND_OK',1);
    } else {
        $Servlet->vars('EMAIL_MISSING',1);
    }
    return;
}

sub DeleteListedOrders {
    my $self = shift;
    my ($Servlet) = @_;

    return if ! $Servlet->params('ListObjectID');

    RunBatchProcess('DeleteOrders', sub {

        my $MasterObject = $Servlet->object;
        my $Shop = $MasterObject->getSite;
        my $Form = $Servlet->form;
        my $hValues = $Form->form($MasterObject, 'ListedObjects');
        my @ListObjects = map { $Shop->loadObject($_, 'CustomerOrder') }
                          grep { ExistsObject($_) }
                          map { $_->{'ListObjectID'} }
                          @{$hValues->{'ListObjectIDs'}};

        # filter elements wich are not archived or archived and cancelled
        my @DeleteObject;
        foreach my $Order (@ListObjects){
            my $ArchivedOn = $Order->get('ArchivedOn');
            my $CancelledOn = $Order->get('CancelledOn');
            push @DeleteObject,$Order if (($ArchivedOn) or ($CancelledOn)) ;
            $Servlet->vars('NOT_ARCHIVED_OR_CANCELLED',1) unless $ArchivedOn or $CancelledOn;
            $Servlet->vars($Order->get('ID').'_NOT_ARCHIVED_OR_CANCELLED',1) unless $ArchivedOn or $CancelledOn;
        }

        @ListObjects = @DeleteObject;

        foreach my $Index ( 0 .. scalar(@ListObjects)-1 ) {
            my $Object = $ListObjects[$Index];
            next if $Servlet->hasPermission($Object, 'Delete');

            $Form->addFormError( {
                Reason => 'PermissionDenied',
                Index => $Index,
                LoopName => 'ListObjectIDs',
                Form => 'ListedObjects',
                Name => 'ListObjectID'
            } );
        }
        $Form->executeFormError;

        # delete only if accepted if confirmation is used
        if (scalar @ListObjects and $Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
            $Servlet->vars('DeleteObjects', [ map { $_->tleHash } @ListObjects]);
            $Servlet->vars('DELETE',1);
            return;
        } else {
            # else reset delete
            $Servlet->vars('DELETE',0);
        }

        # now delete all orders and editable orders before
        Transaction( sub {
            foreach my $Order (@ListObjects){
                my $EditableOrder = GetEditableOrder($Order);
                $EditableOrder->delete if defined $EditableOrder;
                $Order->delete;
            }
        } );
    });

    return;
}

sub _getPrivateExportFileName {
    my $self    = shift;
    my ($Servlet, $XML) = @_;
    my $User = $Servlet->user;

    my $RequestedFileName = $XML ? $Servlet->params('ExportXMLFileName') : $Servlet->params('ExportFileName');
    $RequestedFileName = $XML ? 'orders.xml' : 'orders.txt' unless $RequestedFileName;

    # Check that the given filename doesn't contain a path
    my ($FilePath, $FileNameWithoutExtension, $FileExtension) = fsplit($RequestedFileName);
    GetLog('BUGTRACKER::AD6447')->debug("Requested filename $RequestedFileName for orderexport was manipulated by user ".$User->pathString) if $FilePath;

    my $FileName = $FileNameWithoutExtension.$FileExtension;

    my $Directory = $User->get('PrivatePath');

    return wantarray
        ? ("$Directory/$FileName", $FileName)
        : "$Directory/$FileName";
}

sub ExportCustomerOrders {
    my $self = shift;
    my ($Servlet) = @_;
    return if !$Servlet->params('ListObjectID');

    my $MasterObject = $Servlet->object;
    my $Shop = $MasterObject->getSite;
    my $Form = $Servlet->form;
    my $hValues = $Form->form($MasterObject, 'ListedObjects');

    my @ListObjects;
    foreach my $hListObjectID (@{$hValues->{'ListObjectIDs'}}){
        my $ListObject = $Shop->loadObject( $hListObjectID->{'ListObjectID'}, 'CustomerOrder' );
        $Servlet->testPermission($ListObject, 'Save');
        push @ListObjects,$ListObject;
    }

    my $FileName = $self->_getPrivateExportFileName($Servlet);
    ExportOrdersAsText( \@ListObjects, $FileName, {
        'locale' => $Servlet->locale,
        'encoding' => $Servlet->params('ExportEncoding'),
        'linebreaks' => $Servlet->params('ExportLineBreak'),
    });

    $Servlet->vars( 'Redirect', 1 );
}

sub ViewExportResult {
    my $self    = shift;
    my ($Servlet) = @_;

    my ($PathFileName, $FileName) = $self->_getPrivateExportFileName($Servlet);
    my $Encoding = $Servlet->params('ExportEncoding');
    $Encoding = 'cp1252' unless $Encoding;

    my $rContent= GetFileContent($PathFileName, {'binary'=>1});
    my $Response = $Servlet->response;
    $Response->headers->content_type( "text/plain; charset=$Encoding" );
    $Response->headers->push_header( 'Content-Disposition' => "attachment; filename=$FileName" );

    $Response->content( $$rContent );
    $Servlet->params('ExitAfterEvent',1);
    return;
}

sub ExportCustomerOrdersXML {
    my $self = shift;
    my ($Servlet) = @_;
    return if !$Servlet->params('ListObjectID');

    my $MasterObject = $Servlet->object;
    my $Shop = $MasterObject->getSite;
    my $Form = $Servlet->form;
    my $hValues = $Form->form($MasterObject, 'ListedObjects');

    my @ListObjects;
    foreach my $hListObjectID (@{$hValues->{'ListObjectIDs'}}){
        my $ListObject = $Shop->loadObject( $hListObjectID->{'ListObjectID'}, 'CustomerOrder');
        $Servlet->testPermission($ListObject, 'Save');
        push @ListObjects,$ListObject;
    }

    my $FileName = $self->_getPrivateExportFileName($Servlet, 1);
    my $Driver = DE_EPAGES::Order::API::XML::CustomerOrderExport->new;
    $Driver->exportXML($FileName, { 'locale' => $Servlet->locale, 'encoding' => 'utf-8'}, $Servlet->object, \@ListObjects);

    $Servlet->vars( 'RedirectXML', 1 );
}

sub ViewExportXMLResult {
    my $self    = shift;
    my ($Servlet) = @_;

    my ($PathFileName, $FileName) = $self->_getPrivateExportFileName($Servlet, 1);

    my $rContent= GetFileContent($PathFileName, {'binary'=>1});
    my $Response = $Servlet->response;
    $Response->headers->content_type( 'application/xml; charset=utf-8' );
    $Response->headers->push_header( 'Content-Disposition' => "attachment; filename=$FileName" );

    $Response->content( $$rContent );
    $Servlet->params('ExitAfterEvent',1);
    return;
}

sub ExportCustomerOrderToXML {
    my $self = shift;
    my ($Servlet) = @_;
    my $Object = $Servlet->object;

    my @ListObjects = ($Object);

    my ($PathFileName, $FileName) = $self->_getPrivateExportFileName($Servlet, 1);
    my $Driver = DE_EPAGES::Order::API::XML::CustomerOrderExport->new;
    $Driver->exportXML($PathFileName, { 'locale' => $Servlet->locale, 'encoding' => 'utf-8'}, $Servlet->object, \@ListObjects);

    my $rContent= GetFileContent($PathFileName, {'binary'=>1});
    my $Response = $Servlet->response;
    $Response->headers->content_type( 'application/xml; charset=utf-8' );
    $Response->headers->push_header( 'Content-Disposition' => "attachment; filename=$FileName" );

    $Response->content( $$rContent );
    $Servlet->params('ExitAfterEvent',1);
    return;
}

sub CreateEditableOrder ($$) {
    my $self = shift;
    my ($Servlet, $Order) = @_;

    my $EditableOrder = GetEditableOrder($Order);

    # clone order if editable order does not yet exist
    unless( defined $EditableOrder ) {
        Transaction( sub {
            $EditableOrder = $Order->clone({
                'Parent'        => $Order->get('Customer')->folder('EditableOrders'),
                'Inherit'       => 1,
                'OriginalOrder' => $Order
            }, 1);
        });
    }

    # show the page in the edit mode
    $Servlet->params('ObjectID',$EditableOrder->id);

    return $EditableOrder;
}

sub NewOrder {
    my $self = shift;
    my ($Servlet) = @_;
    my $Customer = $Servlet->object;
    my $Shop = $Customer->getSite;

    #look for users belongs to customer
    my $aCustomerUser = GetPKeyCustomerUsersByCustomerID($Customer->id);

    my $Order;
    eval {
        RunBatchProcess('CreateOrders', sub {
            if (scalar @$aCustomerUser > 0){#one or more users belongs to customer
                #only look for first user
                my $UserID = (@$aCustomerUser)[0]->{'UserID'};
                my $User = ExistsObject($UserID) ? $Shop->loadObject($UserID, 'User') : undef;
                if (defined $User and
                    defined $User->get('CurrencyID') and
                    ExistsShopCurrency($Customer->get('SiteID'), $User->get('CurrencyID')) ){
                    #user exists has a currencyid that belongs to shop, order created with currencyid
                    $Order = _CreateOrder($Customer, {'CurrencyID' => $User->get('CurrencyID'),
                                                      'User'       =>  $User});
                }
                else{#currencyid doesnt exists or dont belongs to shop or $User doesnt exists
                    $Order = _CreateOrder($Customer, {'User' => $User});
                }
            } else {#no user exists for customer
                $Order = _CreateOrder($Customer, {'User' => undef});
            }
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless ElementOf($Error->code, qw (NoShipping NoPayment));
        $Servlet->params($Error->code, 1);
        $Servlet->params('ViewAction', 'MBO-ViewGeneral');
        return $Customer;
    }

    $Servlet->params('ViewAction', 'MBO-ViewGeneral');
    my $EditableOrder = $self->CreateEditableOrder($Servlet, $Order);

    return $EditableOrder;
}

sub ViewPrint {
    my $self = shift;
    my ($Servlet) = @_;
    my $Object = $Servlet->object;
    my $Container = $Object->get('LineItemContainer');
    $Servlet->setLocale( $Container->get('LocaleID'), 0 );
}

sub ViewPDF {
    my $self = shift;
    my ($Servlet) = @_;

    my $OrderDocument = $Servlet->object;
    my $hPdfContent = eval { GenerateOrderDocumentPDF($OrderDocument) };
    if(ExistsError()){
        my $Error = GetError();
        LogError($Error);
        $Servlet->form->executeFormError({
            'Name' => 'ViewPdf',
            'Reason' => 'GeneratePdfFailed',
            'ViewAction' => 'MBO-ViewPDF',
        });
    }

    my $PdfContent = $hPdfContent->{'pdf'};
    my $PdfFileName = $hPdfContent->{'filename'};

    my $Response = $Servlet->response;
    $Response->content( $PdfContent);
    $Response->header( 'Content-Type' => 'application/pdf',
                       'Content-Disposition' => 'attachment; filename="' . $PdfFileName. '"',
                       'Content-Description' => 'order document pdf' );
    $Servlet->vars('ExitAfterEvent',1);
    return;
}

sub SendDocumentAsPDFMail {
    my $self = shift;
    my ($Servlet) = @_;
    my $OrderDocument = $Servlet->object();

    my %JsonStatusMsg = ( 'mailSent' => 'true' );

    # At this point, the $OrderDocument may be a Customer Order, an Invoice, a Packing Slip, a Credit Note, or a
    # Shipping Label. If the merchant clicked sendPDF from the orders page, we want to send the invoice but here
    # we have a CustomerOrder (so we need to make this kind of translation).
    if ( $OrderDocument->instanceOf('CustomerOrder')){
        $OrderDocument = GetLatestOrderInvoice($OrderDocument);
    }
    # After that, what we have here must be an Invoice, a Packing Slip, a Credit Note, or a Shipping Label.

    unless(defined $OrderDocument){
        %JsonStatusMsg = ('mailSent' => 'false', 'reason' => 'DOCUMENT_NOT_FOUND');
    }

    my $Email   = $Servlet->params('To');
    my $Cc      = $Servlet->params('Cc');
    my $Bcc     = $Servlet->params('Bcc');
    my $Subject = $Servlet->params('Subject');

    unless(defined $Email){
        $JsonStatusMsg{'mailSent'} = 'false';
        $JsonStatusMsg{'reason'} = [ 'EMAIL_MISSING' ];
    }

    eval{
        TestMultipleEMailAddresses('Recipient', $Email);
        TestMultipleEMailAddresses('Recipient', $Cc) if defined $Cc;
        TestMultipleEMailAddresses('Recipient', $Bcc) if defined $Bcc;
    };
    if (ExistsError()) {
        $JsonStatusMsg{'mailSent'} = 'false';
        $JsonStatusMsg{'reason'} = [ 'InvalidMailAddress' ];
    }

    unless(defined $Subject){
        $JsonStatusMsg{'mailSent'} = 'false';
        if(exists $JsonStatusMsg{'reason'}){
            push @{$JsonStatusMsg{'reason'}}, 'SUBJECT_MISSING';
        } else {
            $JsonStatusMsg{'reason'} = [ 'SUBJECT_MISSING' ];
        }
    }

    if($JsonStatusMsg{'mailSent'} eq 'true'){
        my $aJsonWarnings = [];
        my $aWarnings = eval {
            BlockWarnings( sub {
                    SendOrderDocumentAsPDFMail($OrderDocument,$Servlet);
                }, [ 'No_Recipients',
                     'Invalid_Recipient',
                     'InactiveMailType',
                     'MISSING_FROM',
                   ]);
        };
        if (ExistsError()){
            LogError(GetError());
            %JsonStatusMsg = ('mailSent' => 'false');
        }

        if(defined $aWarnings and scalar @$aWarnings){
            push @$aJsonWarnings, $_->code() foreach @$aWarnings;
            %JsonStatusMsg = ('mailSent' => 'false', 'reason' => $aJsonWarnings);
        }
    }

    $Servlet->vars('ResponseContent',\%JsonStatusMsg);
    return;
}

sub ViewCustomerOrders {
    my $self = shift;
    my ($Servlet) = @_;

    my $Customer = $Servlet->object;

    #START: show total order amount
    my $aOrders = $Customer->get('Orders');
    my %hTotalOrderAmounts;
    foreach my $Order ( @$aOrders ) {
        next if $Order->get('CancelledOn');

        my $LineItemContainer = $Order->get('LineItemContainer');
        my $CurrencyID = $LineItemContainer->get('CurrencyID');
        my $TotalBeforeTax = $LineItemContainer->get('TotalBeforeTax') // 0;

        if(!exists($hTotalOrderAmounts{$CurrencyID})) {
            $hTotalOrderAmounts{$CurrencyID} = {
                'CurrencyID' => $CurrencyID,
                'Amount' => $TotalBeforeTax,
            };
        }
        else {
            $hTotalOrderAmounts{$CurrencyID}{'Amount'} += $TotalBeforeTax;
        }
    }

    # build proper output
    my @aTotalOrderAmounts;
    # default currency should apear at first position
    if (scalar(keys(%hTotalOrderAmounts)) > 0) {
        my $DefaultCurrencyID = $Customer->getSite()->get('CurrencyID');
        push(@aTotalOrderAmounts,delete($hTotalOrderAmounts{$DefaultCurrencyID}))
         if(exists($hTotalOrderAmounts{$DefaultCurrencyID}));
        push(@aTotalOrderAmounts,map {$hTotalOrderAmounts{$_}} sort keys %hTotalOrderAmounts);
    }
    $Servlet->vars('TotalOrderAmounts', \@aTotalOrderAmounts);
    #END: show total order amount
}

#========================================================================================
# §function     _CreateOrder
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Order = _CreateOrder($Customer, $hVars);
#----------------------------------------------------------------------------------------
# §description  Helper function for creating an empty order for the given customer
#               without using the API function Order::API::Order::CreateOrder,
#               because this API function does check the shipping and payment methods (EPG-23946).
#               Here all shipping methods can be used for creating a MBO order.
#               All payment methods without customer actions can be used for creating a MBO order.
#               The dependency map payment vs. shipping is ignored for creating a MBO order.
#----------------------------------------------------------------------------------------
# §input        $Customer | customer |  object
# §input        $hVars | <ul>
#                   <li>CurrencyID - (optional; default: shop default currency) alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel - (optional; default: customer tax model) gross (1) or net (0) price  - int
#                   <li>User - (optional) customers user or undef - user object
#               </ul> | ref.hash (optional)
# §return       $Order | new customer order | object
#========================================================================================
sub _CreateOrder{
    my ($Customer, $hVars) = @_;

    TestObject( 'Customer', $Customer, 'DE_EPAGES::Customer::API::Object::Customer' );

    $hVars //= {};
    my $Shop = $Customer->getSite();

    my $CurrencyID   = $hVars->{'CurrencyID'} // $Shop->get('CurrencyID');
    my $TaxModel     = $hVars->{'TaxModel'} // $Customer->get('TaxModel');
    my $CustomerUser = $hVars->{'User'} // undef;

    # get customer addresses
    my $BillingAddress = $Customer->get('BillingAddress');
    Error('NoBillingAddress', {'Customer' => $Customer->get('Path')}) unless defined $BillingAddress;

    # for Unity only weget ShippingAddress from User for cloning this address to the newly created order
    # (remember: we have no ShippingAddress on the Customer!)
    my $ShippingAddress = undef;
    if(defined $CustomerUser and $Shop->get('IsUnityRegistered')){
        $ShippingAddress = $CustomerUser->get('ShippingAddress');
    }

    # get all shipping methods from shop
    my $ShippingMethods = $Shop->get('ShippingMethods');
    Error('NoShipping') unless scalar @$ShippingMethods;

    # get all payment methods from shop
    # in MBO only not customer requiring payment methods could be choosen
    # the dependencies payment/shipping are ignored
    my @PaymentMethods = grep { not($_->get('PaymentType')->get('IsCustomerRequired')) } @{$Shop->get('PaymentMethods')};
    Error('NoPayment') unless scalar @PaymentMethods;

    # search for the default payment method
    my ($DefaultPaymentMethod) = grep { $_->get('IsDefault') } @PaymentMethods;
    $DefaultPaymentMethod //= $PaymentMethods[0];  # select first one if no method is marked as default
    if (!$DefaultPaymentMethod->usableAtStorefront({'CurrencyID' => $CurrencyID ,'TaxModel' => $TaxModel})) {
        # if not usable at storefront, find another usable one
        foreach my $PaymentMethod (@PaymentMethods) {
            if ($PaymentMethod->usableAtStorefront({'CurrencyID' => $CurrencyID ,'TaxModel' => $TaxModel})) {
                $DefaultPaymentMethod = $PaymentMethod;
                last;
            }
        }
    }

    my %CopyVars = (
        %$hVars,
        'Alias' => GetNextOrderNo($Shop,$Customer), # generate next order number before the transaction to avoid long locks
    );

    # create the order and add the payment to the container
    my $Order;
    Transaction( sub {

        $Order = CreateOrderWithContainer( $Customer, \%CopyVars );
        my $CloneBillingAddress = $BillingAddress->clone({ 'Parent' => $Order, 'Alias' => 'BillingAddress' });
        $Order->set({ 'BillingAddress' => $CloneBillingAddress });

        if(defined $ShippingAddress){
            my $CloneShippingAddress = $ShippingAddress->clone({ 'Parent' => $Order, 'Alias' => 'ShippingAddress' });
            $Order->set({ 'ShippingAddress' => $CloneShippingAddress });
        }

        if($Shop->get('TaxMethod')){
            $Shop->get('TaxMethod')->updateContainerTaxArea($Order->container);
        }else {
            $Order->updateTaxArea;
            $Order->updateTaxAreaDigital if defined $Shop->get('TaxMatrixDigital');
        }
        # insert payment if not already exists
        my $Container = $Order->get('LineItemContainer');
        $DefaultPaymentMethod->addToContainer($Container, {}) unless $Container->get('Payment');

        # set ViewedOn for manually created order
        $Order->set({'ViewedOn' => GetCurrentDateTime()});

    });

    return $Order;
}

sub ViewMultiCheckoutCustomerOrder {
    my $self = shift;
    my ($Servlet) = @_;

    my $Order = $Servlet->object;
    my $Shop = $Order->getSite;

    if ($Shop->get('IsUnityRegistered')) {
        # add languge to redirect URL because we need the order information page in the same
        # language like basket process. The language id from the container is used
        my $UserLanguageID = $Servlet->user->get('LanguageID');
        my $URL = AddLanguageToURL($Order->get('WebUrlUnitySSL'), $UserLanguageID, 0);

        # delete session information in case of an anonymous user
        $Servlet->logout if $Servlet->user->get('IsAnonymous');
        $Servlet->redirect($URL);
    }
}

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::View($Servlet);

    my $Order = $Servlet->object;
    my $Shop = $Order->getSite;

    if ($Shop->get('IsUnityRegistered')) {
        $Servlet->redirect($Order->get('WebUrlUnitySSL'));
    }
}

sub _lastUpdate{
    my $self = shift;
    my ($Order) = @_;

    my $OrderChangeProfile = LoadClassByAlias('CustomerOrder')->lastUpdateProfile('OrderChange');
    $Order->setUpdated($OrderChangeProfile);

    return;
}

1;
