#========================================================================================
# package      DE_EPAGES::Payment::API::Object::PaymentMethod
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for PaymentMethod.
#========================================================================================
package DE_EPAGES::Payment::API::Object::PaymentMethod;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestArray TestFloat TestInteger TestString );
use DE_EPAGES::Core::API::Warning;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Payment::API::Constants qw ( CLASSICCHECKOUT MULTISTEPCHECKOUT );
use DE_EPAGES::Payment::API::Table::PaymentMethodCustomerGroup qw (
    DeletePaymentMethodCustomerGroup
    GetPKeyPaymentMethodCustomerGroupsByPaymentMethodID
);
use DE_EPAGES::Payment::API::Table::PaymentMethodDiscount qw (
    DeletePaymentMethodDiscount
    GetInfoPaymentMethodDiscountsByPaymentMethodID
    GetPKeyPaymentMethodDiscountsByPaymentMethodID
    InsertPaymentMethodDiscount
    UpdatePaymentMethodDiscount
);
use DE_EPAGES::Payment::API::Table::PaymentMethodPriceLimit qw (
    DeletePaymentMethodPriceLimit
    GetInfoPaymentMethodPriceLimitsByPaymentMethodID
    GetPKeyPaymentMethodPriceLimitsByPaymentMethodID
    InsertPaymentMethodPriceLimit
    UpdatePaymentMethodPriceLimit
);
use DE_EPAGES::Payment::API::Table::ShippingPaymentMap qw (
    DeleteShippingPaymentMap
    GetPKeyShippingPaymentMapsByPaymentMethodID
);
use DE_EPAGES::Payment::API::XML::PaymentDeleteHandler;
use DE_EPAGES::Payment::API::XML::PaymentExportDriver;
use DE_EPAGES::Payment::API::XML::PaymentImportHandler;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $PaymentMethod->featureName;
#----------------------------------------------------------------------------------------
# description  Returns the feature name.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName () { return undef; }


#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $PaymentMethod->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'PaymentMethod' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $PaymentMethod->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod = DE_EPAGES::Payment::API::Object::PaymentMethod->insert($hInfo);
# example      my $Class = LoadClassByAlias('PaymentMethod');
#               my $PaymentMethod = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertPaymentMethod' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $PaymentMethod | new object | object
# hook         OBJ_InsertPaymentMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentMethodID - payment method id    - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $Shop = $hInfo->{'Shop'} || $hInfo->{'Parent'}->getSite();

    # increment the usage counter of the payment methods feature
    $Shop->featureIncrementValue('PaymentMethods');

    # test special payment feature
    my $FeatureName = $class->featureName;
    Error( 'FeatureLimitExceeded', {
        Feature => $FeatureName,
        MaxValue => 0,
        Shop => $Shop
    }) if defined $FeatureName and not $Shop->featureMaxValue($FeatureName);

    my $PaymentType = $hInfo->{'PaymentType'};
    if (not $PaymentType->canCreateAdditionalPaymentMethod($Shop)) {
        Error('NoSecondInstanceAllowed', {'Shop' => $Shop});
    }

    my $self = $class->SUPER::insert($hInfo);
    TriggerHook('OBJ_InsertPaymentMethod', {'Object' => $self, 'PaymentMethodID' => $self->id });

    delete $hInfo->{$_} foreach qw ( Shop PaymentType TaxClass IsActivated IsDefault IsHirePurchaseEnabled CashDiscountMode CashDiscount ContainerSubTotalAttrName DoPaymentOnBasket TransactionType IsInTestMode Region WorksWithoutShipping InternalImagePath ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeletePaymentMethod' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeletePaymentMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentMethodID - payment method id    - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeletePaymentMethod', {'Object' => $self, 'PaymentMethodID' => $self->id });

    #--- get current feature value
    my $Shop = $self->getSite();

    LoadClassByAlias('PaymentLevel')->deleteObjects('ByPaymentMethodID', [$self->id]);
    DeletePaymentMethodCustomerGroup($_) foreach @{GetPKeyPaymentMethodCustomerGroupsByPaymentMethodID($self->id)};
    DeletePaymentMethodDiscount($_) foreach @{GetPKeyPaymentMethodDiscountsByPaymentMethodID($self->id)};
    DeletePaymentMethodPriceLimit($_) foreach @{GetPKeyPaymentMethodPriceLimitsByPaymentMethodID($self->id)};
    DeleteShippingPaymentMap($_) foreach @{GetPKeyShippingPaymentMapsByPaymentMethodID($self->id)};
    $self->SUPER::delete;

    #--- update feature value
    $Shop->featureIncrementValue('PaymentMethods', -1);
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdatePaymentMethod' and 'OBJ_AfterUpdatePaymentMethod'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdatePaymentMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentMethodID - payment method id    - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdatePaymentMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentMethodID - payment method id    - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdatePaymentMethod', {'Object'=>$self, 'Values'=> $hValues, 'PaymentMethodID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdatePaymentMethod',  {'Object'=>$self, 'Values'=> $hValues, 'PaymentMethodID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Payment::API::Object::PaymentMethod->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    my $PaymentType = $hInfo->{'PaymentType'};

    return $class->SUPER::default($Parent, $hInfo) unless defined $PaymentType;

    my $DoPaymentOnBasket     = $PaymentType->get('CanPaymentOnOrder') ? 0 : 1;
    my $CanTransactionCapture = $PaymentType->get('CanTransactionCapture');
    my $TransactionType       = $CanTransactionCapture ? 0 : 1;

    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'IsActivated' => 0,
        'IsDefault' => 0,
        'IsHirePurchaseEnabled' => 0,
        'DoPaymentOnBasket' => $DoPaymentOnBasket,
        'TransactionType' => $TransactionType,
        'IsInTestMode' => 0,
        'TaxClass' => $Parent->getSite()->get('TaxMatrix')->defaultTaxClass,
        'CashDiscount' => 0,
        'CashDiscountMode' => 0,
        'WorksWithoutShipping' => 1,
        'Shop' => $Parent->getSite,
    };
}

sub importXML {
    my ($ObjectPackage, $Handler, $hElement, $Class, $hAttributes) = @_;

    my $Parent = $Handler->topID('Object');
    # get shop
    my $Shop = $Parent->getSite() or die 'No Shop found for object '.$Parent->alias;
    $hAttributes->{'Shop'} = $Shop;

    # get tax class
    if (exists $hAttributes->{'TaxClass'}) {
        my $TaxMatrix = $Shop->get('TaxMatrix');
        $hAttributes->{'TaxClass'} = $TaxMatrix->child($hAttributes->{'TaxClass'});
    }
    my $Object = $ObjectPackage->SUPER::importXML($Handler, $hElement, $Class, $hAttributes);

    # delete levels before import, because there is no unique key
    my $PaymentLevelClass = LoadClassByAlias('PaymentLevel');
    $_->delete foreach @{ $PaymentLevelClass->getObjects('ByPaymentMethodID', {
        'PaymentMethodID' => $Object->id
                        })
    };

    return $Object;
}

#========================================================================================
# function     exportXMLAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAttributes = $Object->exportXMLAttributes( $Driver, $hAllAttributes );
#----------------------------------------------------------------------------------------
# description  Returns plain attributes of object to XML export driver. This function
#               removes bit attributes with value 0.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function exportableAttributes> | ref.hash
# return       $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLAttributes {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;
    $hAttributes = $self->SUPER::exportXMLAttributes($Driver, $hAttributes);

    # map TaxClass name to TaxClassID
    my $TaxClass = $self->get('TaxClass');
    $hAttributes->{'TaxClass'} = $TaxClass->alias if defined $TaxClass;
    return $hAttributes;
}

sub exportXMLElements {
    my $self = shift;
    my ($Driver, $allAttributes) = @_;
    $self->SUPER::exportXMLElements($Driver, $allAttributes);
    if ($Driver->exportLevel == $self->class->get('ExportLevel')) {
        DE_EPAGES::Object::API::XML::ObjectExportDriver::ShippingPaymentMapsByPaymentMethod($Driver);
        DE_EPAGES::Object::API::XML::ObjectExportDriver::CustomerGroupsByPaymentMethod($Driver);
        DE_EPAGES::Object::API::XML::ObjectExportDriver::PaymentMethodPriceLimitsByPaymentMethod($Driver);

        my $PaymentLevelClass = LoadClassByAlias('PaymentLevel');
        my $aPaymentLevels = $PaymentLevelClass->getObjects('ByPaymentMethodID', {'PaymentMethodID'=>$self->id});
        $Driver->addObject( 'PaymentLevel', $_ ) foreach @$aPaymentLevels;

    }



    return;
}

sub taxMatrix {
    my $self = shift;
    return $self->getSite()->get('TaxMatrix');
}

#========================================================================================
# function     price
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentPrice = $Object->price( $hVars );
#----------------------------------------------------------------------------------------
# example      $PaymentPrice = $PaymentMethod->price( {
#                   CurrencyID => 'USD',
#                   TaxModel   => $PaymentMethod->getSite()->get('TaxModel')
#               } );
#----------------------------------------------------------------------------------------
# description  Returns the payment price by the given requirements. Needs to be
#               implemented for a new payment method.
#----------------------------------------------------------------------------------------
# return       $PaymentPrice | price of payment | money
#========================================================================================
sub price {
    my $self = shift;
    return 0;
}

#========================================================================================
# function     canDelete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CanDelete = $Object->canDelete($aWarnings);
#----------------------------------------------------------------------------------------
# description  Test if the object can be deleted. In case of the default payment
#               method a warning is added. The default payment method can't be deleted.
#----------------------------------------------------------------------------------------
# input        $aWarnings  | array of Warning Objects | array.ref
# return       $CanDelete  | true if the payment method can be deleted | boolean
#========================================================================================
sub canDelete {
    my $self = shift;
    my ($aWarnings) = @_;

    if ($self->get('IsDefault')) {
        my $Warning = DE_EPAGES::Core::API::Warning->new(
            Code => 'CanNotDeleteDefault',
            Vars => {},
        );
        push @$aWarnings, $Warning;
    }

    return $self->SUPER::canDelete($aWarnings);
}

#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Usable = $PaymentMethod->usableAtStorefront($hVars);
#----------------------------------------------------------------------------------------
# description  Tests if a payment method with the given currency is visible in the storefront.
#               Uses function <function DE_EPAGES::Payment::API::Object::PaymentMethod::price>
#               to evaluate if payment methode has a price (price can be 0).
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $Usable | true if payment method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;

    return 0 unless $self->get('PaymentType')->get('IsActivated');
    eval { $self->price($hVars); };
    if (ExistsError()) {
        my $Error = GetError();
        return 0 if $Error->code eq 'PaymentMethodNoPrice';
        Error($Error);
    }
    return 1;
}

sub addCustomerGroup {
    my $self = shift;
    my ($CustomerGroup ) = @_;

    # security check: paymentmethod and customer should belong to the same shop
    Error('InvalidShop',{ a=>$CustomerGroup->getSite()->id, b=>$self->getSite()->id}) unless $CustomerGroup->getSite()->id == $self->getSite()->id;

    # check if the customergroup is already assigned to the payment method
    my $aCustomerGroups = $self->get('CustomerGroups');
    return if grep { $_->id == $CustomerGroup->id } @$aCustomerGroups;

    # add the customer to the paymentmethod
    push @$aCustomerGroups, $CustomerGroup;

    $self->set({ 'CustomerGroups' => $aCustomerGroups });

    return 1;
}

sub removeCustomerGroup {
    my $self = shift;
    my ($CustomerGroup) = @_;

    # security check: pricelist and customer group should belong to the same shop
    Error('InvalidShop',{ a=>$CustomerGroup->getSite()->id, b=>$self->getSite()->id}) unless $CustomerGroup->getSite()->id == $self->getSite()->id;

    # remove the customer group from the list
    my @CustomerGroups = grep { $_->id != $CustomerGroup->id } @{$self->get('CustomerGroups')};

    # save the new list
    $self->set({ 'CustomerGroups' => \@CustomerGroups });

    return 1;
}

#========================================================================================
# function     levels
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aPaymentLevels = $Object->levels;
#----------------------------------------------------------------------------------------
# description  Returns all payment level objects of that payment method.
#----------------------------------------------------------------------------------------
# return       $aPaymentLevels | list of payment level opbjects | ref.array.object
#========================================================================================
sub levels {
    my $self = shift;

    my $Class = LoadClassByAlias('PaymentLevel');
    return [
        sort {
            $a->get('CurrencyID') cmp $b->get('CurrencyID') ||
            $a->get('LowerBound') <=> $b->get('LowerBound')
        }
        @{$Class->getObjects('ByPaymentMethodID', { 'PaymentMethodID' => $self->id })}
    ];
}

#========================================================================================
# function     getLevels
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahLevels = $Object->getLevels($CurrencyID, $TaxModel);
# syntax       $ahLevels = $Object->getLevels($CurrencyID, $TaxModel, $ExtraPrecision);
#----------------------------------------------------------------------------------------
# description  Return the payment levels for the given currency in the requested tax
#               model.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency  id | char(3)
# input        $TaxModel | tax model (1=gross/including tax, 0=net/without tax) | int
# input        $ExtraPrecision | (optional, default=0) extra precision for net prices
#                           in back-office | boolean
# return       $ahLevels | list of hashes, keys:
#                           BaseValue, LowerBound, UpperBound, Multiplier, PaymentLevelID
#                           | int
#========================================================================================
sub getLevels {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $ExtraPrecision) = @_;
    TestString('CurrencyID', $CurrencyID);
    TestInteger('TaxModel', $TaxModel);
    $ExtraPrecision = 0 unless defined $ExtraPrecision;

    my $PaymentLevelClass = LoadClassByAlias('PaymentLevel');
    my $aLevels = $PaymentLevelClass->getObjects('ByPaymentMethodIDCurrencyID', {
        'PaymentMethodID' => $self->id,
        'CurrencyID' => $CurrencyID
    });

    # convert objects to hashes
    my @Levels = sort { $a->{'LowerBound'} <=> $b->{'LowerBound'} }
        map { $_->get([qw(BaseValue LowerBound UpperBound Multiplier PaymentLevelID)]) }
        @$aLevels;

    # convert net prices to gross
    my $TaxMatrix = $self->getSite()->get('TaxMatrix');
    my $TaxClass = $self->get('TaxClass');
    foreach my $hLevel (@Levels) {
        $hLevel->{'BaseValue'} = $TaxMatrix->net2gross($CurrencyID, $hLevel->{'BaseValue'},
                                                       {'TaxModel' => $TaxModel,
                                                        'TaxClass' => $TaxClass
                                                       }, $ExtraPrecision);
    }

    return \@Levels;
}

#========================================================================================
# function     setLevels
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->setLevels($CurrencyID, $TaxModel, $ahLevels);
#----------------------------------------------------------------------------------------
# description  Sets all payment levels for the given currency.
#               Deletes payment levels that are not included in the list.
#               Calculates net prices from gross prices if $TaxModel is TAX_MODEL_GROSS.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency  id | char(3)
# input        $TaxModel | tax model (1=gross/including tax, 0=net/without tax) | int
# input        $ahLevels | list of hashes, keys:
#                           BaseValue, LowerBound, UpperBound, Multiplier
#                           | int
#========================================================================================
sub setLevels {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $ahLevels) = @_;
    TestString('CurrencyID', $CurrencyID);
    TestInteger('TaxModel', $TaxModel);
    TestArray('ahLevels', $ahLevels);

    # prepare values, make a local copy
    my %LevelsByLowerBound;
    foreach my $hLevel (@$ahLevels) {
        my $LowerBound = $hLevel->{'LowerBound'};
        $LowerBound = 0 unless defined $LowerBound;
        my @Keys = qw(BaseValue LowerBound UpperBound Multiplier);
        $LevelsByLowerBound{$LowerBound}{$_} = $hLevel->{$_} foreach @Keys;
    }

    # convert net prices to gross
    my $TaxMatrix = $self->getSite()->get('TaxMatrix');
    my $TaxClass = $self->get('TaxClass');
    foreach my $hLevel (values %LevelsByLowerBound) {
        $hLevel->{'BaseValue'} = $TaxMatrix->gross2net($CurrencyID, $hLevel->{'BaseValue'},
                                                       {'TaxModel' => $TaxModel,
                                                        'TaxClass' => $TaxClass,
                                                       });

    }

    # update existing levels, delete obsolete ones
    my $PaymentLevelClass = LoadClassByAlias('PaymentLevel');
    my $aPaymentLevels = $PaymentLevelClass->getObjects('ByPaymentMethodIDCurrencyID', {
        'PaymentMethodID' => $self->id,
        'CurrencyID' => $CurrencyID,
    });
    foreach my $PaymentLevel (@$aPaymentLevels) {
        my $LowerBound = $PaymentLevel->get('LowerBound');
        $LowerBound = 0 unless defined $LowerBound;
        if( exists $LevelsByLowerBound{$LowerBound} ) {
            $PaymentLevel->set( $LevelsByLowerBound{$LowerBound} );
            delete $LevelsByLowerBound{$LowerBound};
        }
        else {
            $PaymentLevel->delete;
        }
    }

    # add new levels
    foreach my $hLevel (values %LevelsByLowerBound) {
        $PaymentLevelClass->insertObject({
            'PaymentMethod' => $self,
            'CurrencyID' => $CurrencyID,
            'BaseValue'  => $hLevel->{'BaseValue'},
            'LowerBound' => $hLevel->{'LowerBound'},
            'UpperBound' => $hLevel->{'UpperBound'},
            'Multiplier' => $hLevel->{'Multiplier'},
        });
    }
}
#========================================================================================
# function     template
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Name = $Object->template($Name, $ObjectPageType);
#----------------------------------------------------------------------------------------
# description  This function is used to overwrite the given template for the purpose that
#               each payment method can define there own template (visualisation).
#----------------------------------------------------------------------------------------
# input        $Name | include name | string
# input        $ObjectPageType | object related pagetype | DE_EPAGES::Presentation::API::Object::ObjectPageType
# return       $Name | include name | string
#========================================================================================
sub template {
    my $self = shift;
    my ($Name, $ObjectPageType) = @_;
    return $self->classTemplate($Name, $ObjectPageType);
}

#========================================================================================
# function     setMinPriceValue
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Paymentmethod->setMinPriceValue($CurrencyID, $MinValue);
# example      $Paymentmethod->setMinPriceValue('EUR', 50.00);
# example      $Paymentmethod->setMinPriceValue('EUR', undef);
#----------------------------------------------------------------------------------------
# description  Sets the price limit min value of this payment method.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency code | string
# input        $Value | min price. A value of undef deletes the price. | float
#========================================================================================
sub setMinPriceValue {
    my $self = shift;
    my ($CurrencyID, $Value) = @_;
    TestString('CurrencyID', $CurrencyID);

    if( defined $Value ) {
        TestFloat('Value', $Value);
    }
    # set min value
    my @ExistingValues = grep { $_->{'CurrencyID'} eq $CurrencyID } @{GetInfoPaymentMethodPriceLimitsByPaymentMethodID($self->id)};
    if (scalar @ExistingValues) {
        if (defined $Value or defined $ExistingValues[0]{'MaxPriceValue'} ) {
            UpdatePaymentMethodPriceLimit({ %{$ExistingValues[0]}, 'MinPriceValue'=>$Value});
        } else {
            DeletePaymentMethodPriceLimit($ExistingValues[0]);
        }
    } elsif(defined $Value) {
        InsertPaymentMethodPriceLimit({
            'PaymentMethodID'  => $self->id,
            'CurrencyID' => $CurrencyID,
            'MinPriceValue'  => $Value
        });
    }
}
#========================================================================================
# function     getMinPriceValue
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $PaymentMethod->getMinPriceValue($CurrencyID);
# example      $Value = $PaymentMethod->getMinPriceValue('EUR');
#----------------------------------------------------------------------------------------
# description  Returns the price limit min value of this payment method.
#               Returns undef if the payment method does not have a min value.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency code | string
# return       $Value | price | float
#========================================================================================
sub getMinPriceValue {
    my $self = shift;
    my ($CurrencyID) = @_;
    TestString('CurrencyID', $CurrencyID);

    my @ExistingValues = grep { $_->{'CurrencyID'} eq $CurrencyID } @{GetInfoPaymentMethodPriceLimitsByPaymentMethodID($self->id)};
    return undef unless scalar @ExistingValues;
    my $Price = $ExistingValues[0]{'MinPriceValue'};
    return undef unless defined $Price;

    return $Price;
}

#========================================================================================
# function     setMaxPriceValue
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Paymentmethod->setMaxPriceValue($CurrencyID, $MinValue);
# example      $Paymentmethod->setMaxPriceValue('EUR', 50.00);
# example      $Paymentmethod->setMaxPriceValue('EUR', undef);
#----------------------------------------------------------------------------------------
# description  Sets the price limit max value of this payment method.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency code | string
# input        $Value | min price. A value of undef deletes the price. | float
#========================================================================================
sub setMaxPriceValue {
    my $self = shift;
    my ($CurrencyID, $Value) = @_;
    TestString('CurrencyID', $CurrencyID);

    if( defined $Value ) {
        TestFloat('Value', $Value);
    }
    # set min value
    my @ExistingValues = grep { $_->{'CurrencyID'} eq $CurrencyID } @{GetInfoPaymentMethodPriceLimitsByPaymentMethodID($self->id)};
    if (scalar @ExistingValues) {
        if (defined $Value or defined $ExistingValues[0]{'MinPriceValue'} ) {
            UpdatePaymentMethodPriceLimit({ %{$ExistingValues[0]}, 'MaxPriceValue'=>$Value});
        } else {
            DeletePaymentMethodPriceLimit($ExistingValues[0]);
        }
    } elsif(defined $Value) {
        InsertPaymentMethodPriceLimit({
            'PaymentMethodID'  => $self->id,
            'CurrencyID' => $CurrencyID,
            'MaxPriceValue'  => $Value
        });
    }
}

#========================================================================================
# function     getMaxPriceValue
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $PaymentMethod->getMaxPriceValue($CurrencyID);
# example      $Value = $PaymentMethod->getMaxPriceValue('EUR');
#----------------------------------------------------------------------------------------
# description  Returns the price limit max value of this payment method.
#               Returns undef if the payment method does not have a max value.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency code | string
# return       $Value | price | float
#========================================================================================
sub getMaxPriceValue {
    my $self = shift;
    my ($CurrencyID) = @_;
    TestString('CurrencyID', $CurrencyID);

    my @ExistingValues = grep { $_->{'CurrencyID'} eq $CurrencyID } @{GetInfoPaymentMethodPriceLimitsByPaymentMethodID($self->id)};
    return undef unless scalar @ExistingValues;
    my $Price = $ExistingValues[0]{'MaxPriceValue'};
    return undef unless defined $Price;

    return $Price;
}

#========================================================================================
# function     setPaymentMethodDiscount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Paymentmethod->setPaymentMethodDiscount($CurrencyID, $Value);
# example      $Paymentmethod->setPaymentMethodDiscount('EUR', 5.00);
# example      $Paymentmethod->setPaymentMethodDiscount('EUR', undef);
#----------------------------------------------------------------------------------------
# description  Sets the discount or fee value of this payment method.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency code | string
# input        $Value | price. A value of undef deletes the price. | float
#========================================================================================
sub setPaymentMethodDiscount {
    my $self = shift;
    my ($CurrencyID, $Value) = @_;
    TestString('CurrencyID', $CurrencyID);

    if( defined $Value ) {
        TestFloat('Value', $Value);
    }
    # set price value
    my @ExistingValues = grep { $_->{'CurrencyID'} eq $CurrencyID } @{GetInfoPaymentMethodDiscountsByPaymentMethodID($self->id)};

    if (scalar @ExistingValues) {
        if (defined $Value or defined $ExistingValues[0]{'PriceValue'} ) {
            UpdatePaymentMethodDiscount({ %{$ExistingValues[0]}, 'PriceValue' => $Value});
        } else {
            DeletePaymentMethodDiscount($ExistingValues[0]);
        }
    } elsif(defined $Value) {
        InsertPaymentMethodDiscount({
            'PaymentMethodID'  => $self->id,
            'CurrencyID' => $CurrencyID,
            'PriceValue'  => $Value
        });
    }
}

#========================================================================================
# function     getPaymentMethodDiscount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $PaymentMethod->getPaymentMethodDiscount($CurrencyID);
# example      $Value = $PaymentMethod->getPaymentMethodDiscount('EUR');
#----------------------------------------------------------------------------------------
# description  Returns the discount or fee price value of this payment method.
#               Returns undef if the payment method does not have a value.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency code | string
# return       $Value | price | float
#========================================================================================
sub getPaymentMethodDiscount {
    my $self = shift;
    my ($CurrencyID) = @_;
    TestString('CurrencyID', $CurrencyID);

    my @ExistingValues = grep { $_->{'CurrencyID'} eq $CurrencyID } @{GetInfoPaymentMethodDiscountsByPaymentMethodID($self->id)};
    return undef unless scalar @ExistingValues;
    my $Price = $ExistingValues[0]{'PriceValue'};
    return undef unless defined $Price;

    return $Price;
}
#========================================================================================
# function     getProviderName
# state        public
#----------------------------------------------------------------------------------------
# syntax       getProviderName( $LanguageID );
#----------------------------------------------------------------------------------------
# description  return the name of the payment provider
#----------------------------------------------------------------------------------------
# input        $LanguageID | language id | integer
# return       $ProviderName | name of payment provider | string
#========================================================================================
sub getProviderName {
     my $self = shift;
    my ($LanguageID) = @_;

    my $PaymentType = $self->get('PaymentType');
    my $ProviderName = $PaymentType->get('Provider', $LanguageID);
    if(not isValid($ProviderName)){
      $ProviderName = $PaymentType->get('NameOrAlias', $LanguageID);
    }

    return $ProviderName;

}
#========================================================================================
# function     getAlternativeCheckoutButtonInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       getAlternativeCheckoutButtonInfo( $Basket );
#----------------------------------------------------------------------------------------
# description  return hash with information about the alternative checkout button for this paymentmethod.
#               returns undef in this default implementation. Has to be implemented by
#               Paymentmethods with alternative checkout buttons.
#----------------------------------------------------------------------------------------
# input        $Basket | basket object | object
# return       info hash | refhash
#========================================================================================
sub getAlternativeCheckoutButtonInfo {
    my $self = shift;
    my ($Basket) = @_;
    return;
}
#========================================================================================
# function     getPaymentMethodName
# state        public
#----------------------------------------------------------------------------------------
# syntax       getPaymentMethodName( $LanguageID );
#----------------------------------------------------------------------------------------
# description  return the name of the payment method
#----------------------------------------------------------------------------------------
# input        $LanguageID | language id | integer
# return       name of payment method | string
#========================================================================================
sub getPaymentMethodName {
    my $self = shift;
    my ($LanguageID) = @_;

    my $PaymentType = $self->get('PaymentType');

    my $UseNameOrAlias = 0;
    if ($PaymentType->class()->existsAttribute('NameAsPaymentMethodName') and $PaymentType->get('NameAsPaymentMethodName')){
        $UseNameOrAlias = 1;
    };

    # Standart payments should use NameOrAlias too.
    if($self->class()->get('Package') =~ /^DE_EPAGES::PaymentTypes/){
        $UseNameOrAlias = 1;
    }

    if($UseNameOrAlias){
        return $self->get('NameOrAlias', $LanguageID);
    }else{
        return $self->alias();
    }

}
#========================================================================================
# function     canBeDefaultPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       canBeDefaultPayment( );
#----------------------------------------------------------------------------------------
# description  check if this payment method can be used as default payment.
#----------------------------------------------------------------------------------------
# return       $CanBeDefaultPayment | can be default payment | boolean
#========================================================================================
sub canBeDefaultPayment {
    my $self = shift;

    my $Shop = $self->get('Shop');
    my $OrderProcess = $Shop->get('OrderProcess');
    my $PaymentType = $self->get('PaymentType');

    return 0 if ( not $PaymentType->get('CanBeDefaultPayment') );

    if( $OrderProcess eq CLASSICCHECKOUT ){
        return $PaymentType->get('CanDoClassicCheckout');
    }elsif( $OrderProcess eq MULTISTEPCHECKOUT ){
        return $PaymentType->get('CanDoMultiStepCheckout');
    }
}

1;
