#========================================================================================
# package      DE_EPAGES::Presentation::API::Template
# base         DE_EPAGES::Dictionary::API::Template
# state        public
#----------------------------------------------------------------------------------------
# description  This module provides page type based template processing, including language
#               dependend keywords.
#----------------------------------------------------------------------------------------
# example      use DE_EPAGES::Presentation::API::Template qw (ProcessPage);
#               use DE_EPAGES::Presentation::API::PageType qw (LoadPageTypeByAlias);
#               use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath);
#
#               my $Object = LoadObjectByPath('/Shops/DemoShop');
#               my $PageType = LoadPageTypeByAlias('Details');
#               my $myContent = ProcessPage($Object, $PageType, 'en', {});
#========================================================================================
package DE_EPAGES::Presentation::API::Template;
use base qw( DE_EPAGES::Dictionary::API::Template );

use strict;
our @EXPORT_OK = qw(
    ProcessPage
    GetEmptyTemplateFilePath
    GetEmptyTemplateName
    GetTLEProcessor
    Translate
    GetAllInfoTemplateType
);

use Time::HiRes qw (time);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( GetInfoCartridge );
use DE_EPAGES::Core::API::Error qw ( Error VarsOnError ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( ExistsFile RealPath fsplit CorrectPathSlashes );
use DE_EPAGES::Core::API::Log qw ( LogDebug GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( IsA Memoize );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestString );
use DE_EPAGES::Core::API::Statistics qw (AddStatistic);
use DE_EPAGES::DataCache::API::Config qw ( ConfigCacheSize );
use DE_EPAGES::DataCache::API::Cache qw ( GetCache );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Installer::API::Patch qw ( IsPatchRunning );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadClassByAlias );
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID GetCurrentLanguageID );
use DE_EPAGES::Object::API::Table::Language qw ( GetInfoLanguage );
use DE_EPAGES::Object::API::Template  qw ( ProcessFileTemplate );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

use constant INCLUDE_DEBUG_NO   => 0;
use constant INCLUDE_DEBUG_HTML => 1;
use constant INCLUDE_DEBUG_JS   => 2;
use constant INCLUDE_DEBUG_CSS  => 3;

my @COMMENT_TAGS = (
    [],
    [ '<!--', '-->' ],
    [ '/*', '*/' ],
    [ '/*', '*/' ],
);

sub ErrorDefinition {
    return {
        'NoTemplateFile' => 'Include template #Template not found (#CartridgeTemplate or #StoreTemplate)',
    };
}

#========================================================================================
# function     GetTLEProcessor
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TLEProcessor = GetTLEProcessor();
# syntax       $TLEProcessor = GetTLEProcessor($CacheName,$PackageName);
# example      $TLEProcessor = GetTLEProcessor('SitePageProcessor',__PACKAGE__);
#----------------------------------------------------------------------------------------
# description  Returns an instance of a TLEProcessor. If possible the object will be
#               retrieved from cache otherwise instantiated.
#----------------------------------------------------------------------------------------
# input        $CacheName   | (optional, default: 'PageProcessor')
#                               name of the cache entry  | string
# input        $PackageName   | (optional, default: 'DE_EPAGES::TLE::API::BaseProcessor')
#                               package name | string
# return       $TLEProcessor | an instance of TLEProcessor | object
#========================================================================================
sub GetTLEProcessor {
    my ($CacheName, $PackageName) = @_;

    $CacheName = 'PageProcessor' unless $CacheName;
    $PackageName = 'DE_EPAGES::TLE::API::BaseProcessor' unless defined $PackageName;

    my $DBI = GetCurrentDBHandle();
    my $Cache = GetCache($DBI->cacheKey, 'TLEProcessor');
    my $TLEProcessor;
    if ($Cache->{$CacheName}) {
        $TLEProcessor = $Cache->{$CacheName};
        $TLEProcessor->reset if $TLEProcessor->can('reset');
    } else {
        $TLEProcessor = $PackageName->new;
        $Cache->{$CacheName} = $TLEProcessor;
        TriggerHook('TLEProcessorRegistration', { 'Processor'=>$TLEProcessor });
    }
    return $TLEProcessor;
}

#========================================================================================
# function     ProcessPage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Content = ProcessPage($Object, $ObjectPageType, $LanguageCode, $hVars, $TLEProcessor, $IncludeHandler, $TemplateName);
# example      my $PageType = LoadPageTypeByAlias( 'SF-Category' );
#               my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $Category );
#               {
#                   local $DE_EPAGES::Object::API::Language::LANGUAGEID = GetPKeyLanguageByCode('en');
#                   print ProcessPage( $Category, $ObjectPageType, 'en', $Category->tleHash );
#               }
#----------------------------------------------------------------------------------------
# description  Localizes a template by replacing localization tags, then replaces
#               embedded placeholders (TLE variables).
#               Returns the complete page content.
#               Note: the parameter $LanguageCode specifies the language for localization
#               tags in the template , i.e. static content.
#               The language for dynamic content, such as attributes of type
#               LocalizedString is specified by the global variable
#               $DE_EPAGES::Object::API::Language::LANGUAGEID.
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# input        $ObjectPageType | ObjectPageType object | object
# input        $LanguageCode | language code | string
# input        $hVars | template vars (vars and loops) | ref.(hash.array.)*string
# input        $TLEProcessor | (optional) template processor | object
# input        $IncludeHandler | (optional) include handler | object
# input        $TemplateName | starting template (default: Page) | string
# input        $DefaultFormatting | (optional) set formatting - default value is 'html'  | string
# return       $Content | replaced content | string
#========================================================================================
sub ProcessPage ($$$;$$$$$) {
    my ($Object, $PageType, $LanguageCode, $hVars, $TLEProcessor, $IncludeHandler, $TemplateName, $DefaultFormatting) = @_;
    TestObject('Object', $Object, 'DE_EPAGES::Object::API::Object::Object');
    TestObject('ObjectPageType', $PageType, 'DE_EPAGES::Presentation::API::Object::ObjectPageType');
    TestString('LanguageCode', $LanguageCode);

    my $StartTime = time;
    $hVars = {} unless defined $hVars;

    $TLEProcessor = GetTLEProcessor() unless defined $TLEProcessor;
    if (not defined $IncludeHandler) {
        $IncludeHandler = __PACKAGE__->new;
        $TLEProcessor->registerHandler('FunctionHandler', $IncludeHandler, 'INCLUDE');
        $TLEProcessor->registerHandler('FunctionHandler', $IncludeHandler, 'IsEmpty');
        $TLEProcessor->registerHandler('FunctionHandler', $IncludeHandler, 'TRANSLATE');
    }
    $TemplateName = 'Page' unless defined $TemplateName;

    my $hTemplate = $PageType->template($TemplateName, $Object );
    $TLEProcessor->setContext( { 'DefaultFormatting' => defined $DefaultFormatting ? $DefaultFormatting : 'html' });
    $TLEProcessor->setContext( { 'PageTypeName' => $PageType->name });
    $TLEProcessor->setContext( { 'PageType' => $PageType });
    $TLEProcessor->setContext( { 'Object' => $Object });
    $TLEProcessor->setContext( { 'ObjectID' => $Object->id });
    $TLEProcessor->setContext( { 'ObjectPath' => $Object->get('Path') });
    my $FileName = CorrectPathSlashes($IncludeHandler->templateFileName($hTemplate));
    my $result = ProcessFileTemplate($FileName, $LanguageCode, $hVars, $TLEProcessor, $IncludeHandler);
    my $usedtime = time() - $StartTime;
    if (GetLog(__PACKAGE__ . '::INCLUDE')->is_debug) {
        my (undef, undef, $Extension) = fsplit($FileName);
        if($Extension eq '.js' or $Extension eq '.css'){
          $result .= sprintf "\n/* END OF PAGE %s %.3f seconds (PageType: %s) */\n", $FileName,$usedtime, $PageType->name;
        } else {
          $result .= sprintf "\n<!-- END OF PAGE %s %.3f seconds (PageType: %s) -->\n", $FileName,$usedtime, $PageType->name;
        }
    }
    if ($TemplateName eq 'Page'){
        AddStatistic('Template_Processing',$usedtime);
    }
    return $result;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $Handler = DE_EPAGES::Presentation::API::Template->new(%options);
#----------------------------------------------------------------------------------------
# description  Simple constructor.
#----------------------------------------------------------------------------------------
# input        %options | options | hash
# return       $Handler | template handler | DE_EPAGES::Presentation::API::Template
#========================================================================================
sub new {
    my $class = shift;
    my (%options) = @_;
    $options{'disableDictionaryStat'} = ConfigCacheSize('PageCache', 'DisableCtmplStat') unless defined $options{'disableDictionaryStat'};
    return $class->SUPER::new(%options);
}

#========================================================================================
# function     INCLUDE
# state        public
#----------------------------------------------------------------------------------------
# syntax       #INCLUDE(#Name, #NoDebug)
#----------------------------------------------------------------------------------------
# example      #INCLUDE("Header") ... #INCLUDE("Footer")
# example      #INCLUDE("StyleSheet", "NoDebug")
#----------------------------------------------------------------------------------------
# description  Returns processed content of the include template. If the debugging
#               mode for this function is enabled, an html comment wraps result
#               (includes file name and processing time). WithDebugInfo strings can be:
#               "NoDebug", "JsDebug", "CssDebug" or "HtmlDebug".
#----------------------------------------------------------------------------------------
# input        #TemplateName | name of the template | string
# input        #WithDebugInfo | type of the debug info to include | string
# return       $result | final template | string
#========================================================================================
sub INCLUDE {
    my $self = shift;
    my ($Processor, $raParams) = @_;
    # Vars is only used by the InlineEditor, not directly by INCLUDE.
    my ($TemplateName, $WithDebugInfo, $Vars) = @$raParams;
    TestString( 'TemplateName', $TemplateName );

    my $StartTime = time;

    # disable debug for style sheets or other
    my $WithDebug = $Processor->{'Context'}{'DefaultFormatting'} eq 'js' ? INCLUDE_DEBUG_JS : INCLUDE_DEBUG_HTML;
    if (defined $WithDebugInfo) {
        $WithDebug = INCLUDE_DEBUG_NO   if $WithDebugInfo eq 'NoDebug';
        $WithDebug = INCLUDE_DEBUG_JS   if $WithDebugInfo eq 'JsDebug';
        $WithDebug = INCLUDE_DEBUG_CSS  if $WithDebugInfo eq 'CssDebug';
        $WithDebug = INCLUDE_DEBUG_HTML if $WithDebugInfo eq 'HtmlDebug';
    }

    # get new include file name
    my $Context = $Processor->getContext();
    my $LanguageIdentifier = $Context->{'LanguageIdentifier'};
    my $PageType = $Context->{'PageType'};

    # get the file name for the Include
    my $Object = $Processor->tle('Object');
    $Object = LoadRootObject() unless IsA( $Object, 'DE_EPAGES::Object::API::Object::Object');
    my $hTemplate = $PageType->template($TemplateName, $Object);
    my $IncludeFileName = $self->templateFileName($hTemplate);

    # process include
    my $CallerFileName = $Processor->getContext()->{'TemplateFileName'};
    $Processor->setContext( {'TemplateFileName'=> $IncludeFileName });
    my $cIncludeCode = $self->byteCodeByFileName($IncludeFileName, $LanguageIdentifier);
    my $result = $Processor->replaceTLEs($cIncludeCode, $Vars);
    $Processor->setContext( {'TemplateFileName'=> $CallerFileName });
    if ($WithDebug != INCLUDE_DEBUG_NO and GetLog->is_debug) {
        if(ShowOnlyFile()) {
            $result = sprintf "\n%s %s %s\n%s\n%s %s %s\n",
            $COMMENT_TAGS[$WithDebug][0], # <!--
            CorrectPathSlashes($IncludeFileName),
            $COMMENT_TAGS[$WithDebug][1], # -->
            $result,
            $COMMENT_TAGS[$WithDebug][0], # <!--
            CorrectPathSlashes($IncludeFileName), #file
            $COMMENT_TAGS[$WithDebug][1], # -->
        } else {
            $result = sprintf "\n%s BEGIN INCLUDE %s %.3f seconds %s\n%s\n%s END INCLUDE %s %s\n",
            $COMMENT_TAGS[$WithDebug][0], CorrectPathSlashes($IncludeFileName), time() - $StartTime, $COMMENT_TAGS[$WithDebug][1],
            $result,
            $COMMENT_TAGS[$WithDebug][0], CorrectPathSlashes($IncludeFileName), $COMMENT_TAGS[$WithDebug][1];
        }
    }
    return $result;
}

#========================================================================================
# function     IsEmpty
# state        public
#----------------------------------------------------------------------------------------
# syntax       #IsEmpty(#Value)
#----------------------------------------------------------------------------------------
# example      #IsEmpty(#INCLUDE("Footer"))
#----------------------------------------------------------------------------------------
# description  Returns true if the value is empty respectively has only debug comments
#========================================================================================
sub IsEmpty {
    my $self = shift;
    my ($Processor, $raParams) = @_;
    my ($Value) = @$raParams;

    my $Clear = defined $Value ? $Value : '';

    $Clear =~ s/(<!--((?!-->).)*-->)//xsg;
    $Clear =~ s/(\/\*((?!\*\/).)*\*\/)//xsg;

    my $Result = $Clear =~ m/^\s*$/xs ? 1 : 0;

    LogDebug('IsEmpty', {IsEmpty=>$Result, Value=>$Value,} );

    return $Result;
}

#========================================================================================
# function     ShowOnlyFile
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ShowOnlyFile = ShowOnlyFile();
# example      if(ShowOnlyFile()) { ... }
#----------------------------------------------------------------------------------------
# description  Returns debug state for template file: show only the template file name
#               or addional info like process time. To use this add an entry to log4perl
#               conf:
#               log4perl.category.DE_EPAGES::Presentation::API::Template::ShowOnlyFile=DEBUG
#----------------------------------------------------------------------------------------
# return       $showonlyfile | 1 when showing only template file name, else 0 | boolean
#========================================================================================
sub ShowOnlyFile {
    return 1 if GetLog->is_debug;
    return 0;
}

#========================================================================================
# function     dictionaryFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aFiles = $Handler->dictionaryFiles( $FileName, $LanguageIdentifier );
# example      $aFiles = $Handler->dictionaryFiles( 'Template.tmpl', 'de' );
#----------------------------------------------------------------------------------------
# description  Returns a list of dictionary files for a language-neutral template file.
#----------------------------------------------------------------------------------------
# input        $FileName | template file name | string
# input        $LanguageIdentifier | language identifier could be 'en' but also 'en-US' | string
# return       $aFiles | list of dictionary file names | array ref
#========================================================================================
sub dictionaryFiles {
    my $self = shift;
    my ($FileName, $LanguageIdentifier) = @_;
    $FileName = RealPath($FileName);
    # $FileName = d:/epages/Cartridges/DE_EPAGES/Order/Templates/MBO/MBO-Orders.html
    # $FileName = d:/epages/Shared/stores/Store/Templates/DE_EPAGES/Order/Templates/MBO/MBO-Orders.html

    # d:/epages/Cartridges
    my $Cartridges = RealPath($ENV{'EPAGES_CARTRIDGES'});

    # d:/epages/Shared/stores/Store/Templates
    my $TemplateStore = LoadRootObject()->get('TemplateStore');
    my $StorePath  = join '/', RealPath($ENV{EPAGES_STORES}), $TemplateStore, 'Templates';

    my ($Path, $Base) = fsplit($FileName);

    my ($TemplateDirectory, $SubPath);
    if ($Path =~ /^$Cartridges((\/[^\/]+){3})/ix or
        $Path =~ /^$StorePath((\/[^\/]+){3})/ix ) {
        $TemplateDirectory = $1; # /DE_EPAGES/Order/Templates
        $SubPath = $';           # /MBO         #'
    } else {
        die "directory '$Path' not in store ($StorePath) or cartridge directory ($Cartridges)";
    }
    # Dictionary Files fuer de
    # d:/epages/Cartridges/DE_EPAGES/Dictionary/Templates/Dictionary.de.xml
    # d:/epages/Shared/stores/Store/Templates/DE_EPAGES/Dictionary/Templates/Dictionary.de.xml
    # d:/epages/Cartridges/DE_EPAGES/Order/Templates/Dictionary.de.xml
    # d:/epages/Shared/stores/Store/Templates/DE_EPAGES/Order/Dictionary.de.xml
    # d:/epages/Cartridges/DE_EPAGES/Order/Templates/MBO/MBO-Orders.de.xml
    # d:/epages/Shared/stores/Store/Templates/DE_EPAGES/Order/Templates/MBO/MBO-Orders.de.xml
    my @Files = (
        $Cartridges . '/DE_EPAGES/Dictionary/Templates/Dictionary',
        $StorePath  . '/DE_EPAGES/Dictionary/Templates/Dictionary',
        $Cartridges . $TemplateDirectory . '/Dictionary',
        $StorePath  . $TemplateDirectory . '/Dictionary',
        $Cartridges . $TemplateDirectory . $SubPath.'/'.$Base,
        $StorePath  . $TemplateDirectory . $SubPath.'/'.$Base,
    );

    @Files = map { $_ . ".$LanguageIdentifier.xml" } @Files;

    LogDebug( 'dictionaryFiles', \@Files );
    return \@Files;
}

#========================================================================================
# function     templateFileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FileName = $Handler->templateFileName( $hTemplate );
# example      $hTemplate = $PageType->template( 'Page', $Object );
#               $FileName = $Handler->templateFileName( $hTemplate );
#----------------------------------------------------------------------------------------
# description  Returns the path to a template file. If the file does not exist in the
#               store directory
#               ($ENV{EPAGES_STORES}/$StoreName/Templates/$Company/$Cartridge/$FileName),
#               then the default template is taken from the cartridge directory
#               ($ENV{EPAGES_CARTRIDGES}/$Company/$Cartridge/Templates/$FileName).
#----------------------------------------------------------------------------------------
# input        $hTemplate | template info hash, see GetInfoTemplate | ref.hash.string
# return       $FileName | template file name | string
#========================================================================================
sub templateFileName {
    my $self = shift;
    my ($hTemplate) = @_;

    my $RelativeFileName = $hTemplate->{'FileName'};
    my $TemplateStore = LoadRootObject()->get('TemplateStore');

    # EPG-10892: cache the real file name, abusing the TemplateByteCode cache
    my $CacheRef = GetCache($TemplateStore, 'TemplateByteCode');
    my $CacheKey = $hTemplate->{'CartridgeID'} . $RelativeFileName;
    my $TemplateFileName = $CacheRef->{$CacheKey};
    if ($self->force
        or not defined $TemplateFileName
        or not $self->disableDictionaryStat
    ) {
        my $CartridgeDirectory = GetInfoCartridge($hTemplate->{'CartridgeID'})->{'Package'};
        $CartridgeDirectory =~ s/::/\//g;
        $CartridgeDirectory .= '/Templates';

        # look in the directory $EPAGES_STORES/STORE/Templates/COMPANY/CARTRIDGE/Templates
        my $StoreTemplate = join ('/', $ENV{EPAGES_STORES}, $TemplateStore, 'Templates', $CartridgeDirectory, $RelativeFileName);
        if( ExistsFile( $StoreTemplate ) ) {
            $TemplateFileName = $StoreTemplate;
        }
        else {
            # look in the directory $EPAGES_CARTRIDGES/COMPANY/CARTRIDGE/Templates
            my $CartridgeTemplate = join ('/', $ENV{EPAGES_CARTRIDGES}, $CartridgeDirectory, $RelativeFileName);
            if( ExistsFile( $CartridgeTemplate ) ) {
                $TemplateFileName = $CartridgeTemplate;
            }
            elsif ( IsPatchRunning() ) {
                GetLog('ZDT::NoTemplateFile')->info("Include template " . $hTemplate->{'Name'}
                    . " not found ($CartridgeTemplate or $StoreTemplate)");
                $TemplateFileName = GetEmptyTemplateFilePath();
            }
            else {
                Error( 'NoTemplateFile' => {
                    Template => $hTemplate->{'Name'},
                    CartridgeTemplate => $CartridgeTemplate,
                    StoreTemplate => $StoreTemplate
                } );
            };
        }

        $CacheRef->{$CacheKey} = $TemplateFileName;
    }

    return $TemplateFileName;
}

#========================================================================================
# function     GetEmptyTemplateFilePath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FilePath = GetEmptyTemplateFilePath();
#----------------------------------------------------------------------------------------
# description  In case the requested template is not available during the zero downtime
#               patch, we want to return an empty template instead of throwing an exception.
#               This is a helper function for the access on the absolute file path to
#               this empty template.
#----------------------------------------------------------------------------------------
# return       $FileName | The absolute file name for empty template. | string
#========================================================================================
sub GetEmptyTemplateFilePath {
    return $ENV{EPAGES_CARTRIDGES} . "/DE_EPAGES/Presentation/Templates/BasePageType.Empty.html";
}

#========================================================================================
# function     GetEmptyTemplateName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TemplateName = GetEmptyTemplateName();
#----------------------------------------------------------------------------------------
# description  In case the requested template is not available during the zero downtime
#               patch we want to return an empty template instead of throwing an exception.
#               This is a helper function for the access on the name of the empty template.
#----------------------------------------------------------------------------------------
# return       $TemplateName | the name of the empty template | string
#========================================================================================
sub GetEmptyTemplateName {
    return 'Empty';
}

#========================================================================================
# function     perlFileNameStore
# state        private
#----------------------------------------------------------------------------------------
# syntax       $StoreName = $Handler->perlFileNameStore;
#----------------------------------------------------------------------------------------
# description  Returns store name to build the perlfile name of the template.
#----------------------------------------------------------------------------------------
# return       $StoreName | TemplateStore of root object | string
#========================================================================================
sub perlFileNameStore {
    return LoadRootObject()->get('TemplateStore');
}

#========================================================================================
# function     byteCodeByFileName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ByteCode = $Handler->byteCodeByFileName($FileName, $LanguageIdentifier);
#----------------------------------------------------------------------------------------
# description  Gets bytecode of filename used for include of <function ProcessFileTemplate>.
#               This function uses the DAL Cache 'TemplateByteCode'.
#----------------------------------------------------------------------------------------
# input        $FileName | filename of  template | string
# input        $LanguageIdentifier | language identifier could be 'en' but also 'en-US' | string
# return       $ByteCode |  bytecode | perlsub
#========================================================================================
sub byteCodeByFileName {
    my $self = shift;
    my ($FileName, $LanguageIdentifier) = @_;

    my $ByteCode  = eval {
        local $DE_EPAGES::TLE::API::Execute::TEMPLATE_SOURCE = $FileName;
        my $PerlFileName = $self->perlFileName($FileName, $LanguageIdentifier);

        my $TemplateCacheKey = LoadRootObject()->get('TemplateStore');
        my $CacheRef = GetCache($TemplateCacheKey, 'TemplateByteCode');

        my $CodeRef = $CacheRef->{$PerlFileName};
        if ($self->force
            or not defined $CodeRef
            or not $self->disableDictionaryStat
               and not $self->isNewer($FileName, $PerlFileName, $LanguageIdentifier)
        ) {
            # EPG-31840: if there was an error, try to rebuild the template once
            eval {
                $CodeRef = $self->SUPER::byteCodeByFileName($FileName, $LanguageIdentifier);
            };
            if(ExistsError() && !$self->force) {
                my $Error = GetError();
                Error($Error) unless($Error->code eq 'PERL_SYNTAX_ERROR' || $Error->code eq 'NO_SUB');

                local $self->{'force'} = 1;
                GetLog()->info("Attempting to recreate broken template file $FileName");
                $CodeRef = $self->SUPER::byteCodeByFileName($FileName, $LanguageIdentifier);
            }
            $CacheRef->{$PerlFileName} = $CodeRef;
        }
        return $CodeRef;
    };
    VarsOnError({'TemplateFileName' => $FileName});
    return $ByteCode;
}

#========================================================================================
# function     Translate
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Translation = Translate($Key, $LanguageID, $Cartridge);
# example      $Translation = Translate('Login',undef,'DE_EPAGES::Customer');
#----------------------------------------------------------------------------------------
# description  translates a given keyword
#----------------------------------------------------------------------------------------
# input        $Key | keyword to translate | string
# input        $LanguageID | (optional: default: current language) language id | integer
# input        $Cartridge | (optional: default='DE_EPAGES::Presentation')
#                           cartridge where translation is defined | string
# return       $Translation | translation if found or keyword with { } | string
#========================================================================================
sub Translate ($;$$) {
    my ($Key, $LanguageID, $Cartridge) = @_;
    $LanguageID //= GetCurrentLanguageID();

    my $DialectCode = GetInfoLanguage($LanguageID)->{'Dialect'};
    return _FastTranslate( $Key, $Cartridge, $LanguageID, $DialectCode );
}

# Cache dictionary values.
# Caveats:
# * if the dictionary changes, the values are not updated automatically
# * if the dictionary is overloaded for a special store, the cached value may
#   be randomly chosen from that store or fron the default dictionary
*_FastTranslate = Memoize( sub {
    my ($Key, $Cartridge, $LanguageID, $DialectCode) = @_;
    $Cartridge //= 'DE_EPAGES::Presentation';
    $Cartridge =~ s/::/\//g;

    my $LanguageCode = GetCodeByLanguageID($LanguageID);
    my $LanguageIdentifier = defined $DialectCode ? "$LanguageCode-$DialectCode" : $LanguageCode;
    my $TemplateFile = $ENV{'EPAGES_CARTRIDGES'}.'/'.$Cartridge.'/Templates/dummy.html';
    my $hDictionary = __PACKAGE__->new->dictionary($TemplateFile, $LanguageIdentifier);
    return $hDictionary->{$LanguageCode}->{$Key} // "{$Key}";
} );

#========================================================================================
# function     GetAllInfoTemplateType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahInfo = GetAllInfoTemplateType();
# example      @$ahInfo = @{GetAllInfoTemplateType()};
#----------------------------------------------------------------------------------------
# description  Returns the attributes of all objects of class "TemplateType" as hash.
#----------------------------------------------------------------------------------------
# input        $CartridgeID | cartridge id (optional) | integer
# return       $ahInfo | template type info with keys:<ul>
#                        <li>TemplateTypeID - template type id - integer</li>
#                        <li>Name - template type name - string</li>
#                        <li>ObjectID - object id - integer</li>
#                        <li>FileName - file name - string</li>
#                        <li>CartridgeID - cartridge id - integer</li>
#                        </ul> | ref.array.hash
#========================================================================================
sub GetAllInfoTemplateType {
    my ($CartridgeID) = @_;

    #VARS
    my $ClassTemplateType = LoadClassByAlias('TemplateType');
    my $TemplateTypeObjects;
    if( defined $CartridgeID) {
        #dont have to work with all objects of class "TemplateType"
        $TemplateTypeObjects = $ClassTemplateType->getObjects('ByCartridgeID', [$CartridgeID]);
    }
    else {
        #have to work with all objects of class "TemplateType"
        $TemplateTypeObjects = $ClassTemplateType->objects();
    }
    my @Output;

    #transform object array to an output array
    foreach my $TemplateTypeObject (@$TemplateTypeObjects){
        my $hTempObject = undef;
        $hTempObject->{'TemplateTypeID'} = $TemplateTypeObject->get('TemplateTypeID');
        $hTempObject->{'Name'} = 'TemplateTypeObject: ' . $TemplateTypeObject->alias;
        $hTempObject->{'ObjectID'} = $TemplateTypeObject->get('TemplateTypeID');
        $hTempObject->{'FileName'} = $TemplateTypeObject->get('FileName');
        $hTempObject->{'CartridgeID'} = $TemplateTypeObject->get('CartridgeID');
        push @Output , $hTempObject;
    }
    return \@Output;
}

1;
