#========================================================================================
# package      DE_EPAGES::Presentation::Servlet::PageType
# base         DE_EPAGES::Presentation::Servlet::Permission
# state        public
# description  Extends the permission servlet with processing a page of an object
#========================================================================================
package DE_EPAGES::Presentation::Servlet::PageType;
use base DE_EPAGES::Presentation::Servlet::Permission;

use strict;
use HTTP::Status qw( HTTP_INTERNAL_SERVER_ERROR );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestInteger TestString );
use DE_EPAGES::Core::API::Error  qw( ExistsError );
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::DataCache::API::Cache qw ( GetCache );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObject );
use DE_EPAGES::Permission::API::Permission qw (ActionByName);
use DE_EPAGES::Presentation::API::Object::ObjectPageType;
use DE_EPAGES::Presentation::API::PageType qw ( LoadPageTypeByViewAction LoadPageTypeByAlias );
use DE_EPAGES::Presentation::API::Template qw ( ProcessPage );
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);

#========================================================================================
# function     processContent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->processContent($ViewAction);
#----------------------------------------------------------------------------------------
# description  process a page with the given tle variables. if the no object is given
#               the processContent of the permission servlet will be executed.
#----------------------------------------------------------------------------------------
# input        $ViewAction | view action (optional) | object
# return       $Content | processed template | string
#========================================================================================
sub processContent {
    my $self = shift;
    my ($ViewAction) = @_;
    my $Object         = $self->object;
    $ViewAction = ActionByName($Object->class, $self->params('ViewAction'), $self->vars('IsMobile')) unless defined $ViewAction;

    if ($ViewAction->alias =~ /^JSON/) {
        my $Error = DE_EPAGES::Core::API::Error->new( Code => 'JSON_REQUEST_WRONG_SERVLET', 'Message'=>'PageType Servlet cant handle json requests');
        $self->logUserFault( $Error );
        $self->response->code( HTTP_INTERNAL_SERVER_ERROR );
        return $self->processErrorContent($Error);
    }
    my $PageType       = LoadPageTypeByViewAction( $ViewAction );
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $Object );

    # look if administration part of storefront is enabled
    $self->isEditorMode; # set servlet var

    my $Vars = $Object->tleHash;
    my $ServletVars = $self->vars;
    $Vars->{$_} = $ServletVars->{$_} foreach keys %$ServletVars;
    $Vars->{'INPUT'} = $self->params;
    $Vars->{'PAGETYPE'} = $PageType->tleHash;
    $Vars->{'VIEWACTION'} = $ViewAction->tleHash;

    return ProcessPage($Object, $ObjectPageType, $self->params('Language'), $Vars);
}

#========================================================================================
# function     processErrorOnSpecialPage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->processErrorOnSpecialPage($ViewActionName, $ObjectID);
#----------------------------------------------------------------------------------------
# description  process an error on a special page (e.g. show all errors on startpage)
#               with the given tle variables.
#----------------------------------------------------------------------------------------
# input        $ViewActionName | view action name | string
# input        $ObjectID       | object id | int
# input        $hError         | error data | hash.ref
# return       $Content | processed template | string
#========================================================================================
sub processErrorOnSpecialPage {
    my $self = shift;
    my ($ViewActionName, $ObjectID, $hError) = @_;
    # Test input params.
    TestString('ViewActionName', $ViewActionName);
    TestInteger('ObjectID', $ObjectID);

    # Get object and correct viewaction object.
    my $Object     = LoadObject($ObjectID);
    my $ViewAction = ActionByName($Object->class, $ViewActionName, $self->vars('IsMobile'));

    # Load pagetype by viewaction.
    my $PageType       = LoadPageTypeByViewAction( $ViewAction );
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $Object );

    # look if administration part of storefront is enabled
    $self->isEditorMode; # set servlet var

    my $Vars = $Object->tleHash;
    # Add the error hash on the servlet.
    $self->vars('Error',$hError);
    my $ServletVars = $self->vars;
    $Vars->{$_} = $ServletVars->{$_} foreach keys %$ServletVars;
    $Vars->{'INPUT'} = $self->params;
    $Vars->{'PAGETYPE'} = $PageType->tleHash;
    $Vars->{'VIEWACTION'} = $ViewAction->tleHash;

    return ProcessPage($Object, $ObjectPageType, $self->params('Language'), $Vars);
}

#========================================================================================
# function     doViewAction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->doViewAction;
#----------------------------------------------------------------------------------------
# description  Executes a ViewAction and fills the HTTP response using the template and
#               TLE variables provided by the event handler function.
#               Access permissions are checked before running the event handler.
#========================================================================================
sub doViewAction {
    my $self = shift;

    TriggerHook('UI_BeforeViewAction', { Servlet=>$self });
    my $hParams = $self->params;
    $self->vars('ViewAction' => 'View') unless defined $hParams->{'ViewAction'};
    $self->SUPER::doViewAction;
    TriggerHook('UI_AfterViewAction', { Servlet=>$self });
}

#========================================================================================
# function     historyUri
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->historyUri;
#----------------------------------------------------------------------------------------
# description  returns uri for history.
#----------------------------------------------------------------------------------------
# return       uri | uri for history | string
#========================================================================================
sub historyUri {
    my $self = shift;
    my $ViewAction = $self->params('ViewAction');
    my $Object = $self->object;
    return sprintf('?ViewAction=%s&ObjectID=%d',$ViewAction,$Object->id);
}

#========================================================================================
# function     handlePermissionDenied
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->handlePermissionDenied( $Error );
#----------------------------------------------------------------------------------------
# description  Dispays the login form as a result of an unauthorized view action.
#               Triggers the action ViewLoginForm and sets the followg TLE variables:
#               <ul>
#                   <li>PermissionDenied = 1</li>
#                   <li>PermissionDeniedText = $Error->shortMessage</li>
#               </ul>
#----------------------------------------------------------------------------------------
# input        $Error | error object | object
# return       $Error | handleObjectNotFound if pagetype not defined
#========================================================================================
sub handlePermissionDenied {
    my $self = shift;
    my ($Error) = @_;
    TestObject( 'Error', $Error );

    $self->vars('PermissionDenied', 1);
    $self->vars('PermissionDeniedText', $Error->shortMessage);

    my $OrgViewAction = $self->params->{'ViewAction'} || 'View';
    my $OrgAction = ActionByName( $self->viewObject->class, $OrgViewAction, $self->vars('IsMobile') );
    my $PageType = $OrgAction->get('PageType');
    if (!defined $PageType) {
        my $Site = $self->viewObject->getSite();
        my $Action = ActionByName( $Site->class, 'View',$self->vars('IsMobile') );
        $PageType = $Action->get('PageType');
    }
    unless (defined $PageType) {
        $self->_redirectToUnity();
        return $self->handleObjectNotFound($Error);
    }
    my $LoginViewAction = $PageType->loginViewAction;

    my $hParams = {'Servlet' => $self,
                   'LoginViewAction' => $LoginViewAction};
    TriggerHook('UI_BeforeHandlePermissionDenied', $hParams);

    # redirect to Unity sf to prevent epages6 page processing for unity
    $self->_redirectToUnity() unless $hParams->{'LoginViewAction'} eq 'UnityViewSFLogin';


    $self->doAction( $hParams->{'LoginViewAction'} );
    $self->vars( 'ViewAction', $hParams->{'LoginViewAction'} );

    $self->writeContentOutput($self->processContent);
    return;
}

#========================================================================================
# function     processErrorContent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->processErrorContent( $Error );
#----------------------------------------------------------------------------------------
# description  Generates an error page by processing the page type Error with the System
#               object. Information about the error is added as TLE variables
#               <ul>
#                   <li>Error.Code - error code</li>
#                   <li>Error.ShortMessage - error message</li>
#                   <li>Error.LongMessage - error message with stack trace</li>
#                   <li>Error.Vars - additional information about the error</li>
#               </ul>
#----------------------------------------------------------------------------------------
# input        $Error | error object | object
# return       ProcessPage() | replaced content | string
#========================================================================================
sub processErrorContent {
    my $self = shift;
    my ($Error) = @_;

    # cannot use the PageType version, because it would require a database connection
    return $self->SUPER::processErrorContent($Error) unless GetCurrentDBHandle();

    my $hError = {
        Code => $Error->code,
        GUID => $Error->guid,
        Hash => $Error->hash,
        ShortMessage => $Error->shortMessage,
        LongMessage => $Error->longMessage,
        Vars => $Error->vars,
    };

    my $hParams = {'ErrorCode' => $hError->{'Code'}, 'Site' => $self->site};
    TriggerHook('UI_BeforeErrorProcess',$hParams);

    my $ViewActionName = $self->vars('IsMobile') ? 'MobileSF-'.$hParams->{'ViewAction'} : $hParams->{'ViewAction'};
    my $ObjectID       = $hParams->{'ViewObjectID'};

    $self->_redirectToUnity();

    if(defined $ViewActionName and defined $ObjectID and ($self->params('ViewAction') eq 'View' or $self->params('ViewAction') eq 'MobileSF-View')){
        return $self->processErrorOnSpecialPage($ViewActionName,$ObjectID,$hError);
    }

    my $Params = $Error->code eq 'FALSE_URL_ENCODING' ? $self->defaults : $self->params;

    # process the page type Error with the object System
    my $Object = LoadRootObject();
    my $Vars = $Object->tleHash;
    my $PageType = LoadPageTypeByAlias( 'Error' );
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $Object );

    my $ServletVars = $self->vars;
    $Vars->{$_} = $ServletVars->{$_} foreach keys %$ServletVars;
    $Vars->{'INPUT'} = $Params;
    $Vars->{'Error'} = $hError;

    return ProcessPage($Object, $ObjectPageType, $Params->{'Language'}, $Vars);
}

#========================================================================================
# function     processMonitorContent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hMonitorContentSub = $Servlet->processMonitorContent
#----------------------------------------------------------------------------------------
# description  Returns the functions to generate the content of the monitor page and
#               the finish page.
#               Returns undef if no monitor page shall be used.
#----------------------------------------------------------------------------------------
# return       $hMonitorContent | see <fuction DE_EPAGES::Core::API::Object::MonitorShared::new>
#                   <ul><li>MonitorContentSub - sub to create the content of the monitor page - string</li>
#                       <li>FinishedContentSub - sub to create the content of the finish page - string</li></ul>
#                   | ref.hash
#========================================================================================
sub processMonitorContent {
    my $self = shift;

    my $Language = $self->params('Language');
    my $Site = LoadRootObject(); # monitor page should not be shop-dependend
    my $ContentSub = sub {
        my ($IsContentResponse) = @_;

        my $PageType = LoadPageTypeByAlias( 'Monitor' );
        my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load($PageType, $Site);
        return ProcessPage($Site, $ObjectPageType, $Language, {
            INPUT => $self->params,
            IsContentResponse => $IsContentResponse
        });
    };

    # EPG-33973: pre-process monitor page content to prevent cache corruption.
    # This is not necessary for the finish page, because it will be created after main processing.
    # We cache one monitor pages per database and language.
    my $hCache = GetCache(GetCurrentDBHandle()->cacheKey, 'MonitorContent');
    my $CachedMonitorContent = $hCache->{$Language} //= $ContentSub->(0);

    return {
        MonitorContentSub => sub { return $CachedMonitorContent; },
        FinishedContentSub => sub {

            # Replace the request-dependent variable INPUT BaseLineContentLocation,
            # which is used to jump back to where we came from
            my $Content = $ContentSub->(1);
            my $Location = $self->baseLineContentLocation;
            $Content =~ s/\#INPUT\.BaseLineContentLocation/$Location/gx;

            return $Content;
        },
    };
}

#========================================================================================
# function     _redirectToUnity
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Servlet->_redirectToUnity();
#----------------------------------------------------------------------------------------
# description  Redirects to the WebUrlUnitySSL url if a permission denied error was
#               triggered and we have a sf request and Unity is registered
#========================================================================================
sub _redirectToUnity {
   my $self = shift;
   my $Site = $self->site;

    # Redirect to Unity SF if we have an sf error (session timeout etc.)
    if(($self->_isStoreFrontRequest or $self->_isMobileRequest) ## no critic qw( ProhibitPrivateSubroutineUse)
        and $Site->class->alias eq 'Shop'
        and $Site->get('IsUnityRegistered')) {
        my $ViewObject;
        eval {
            $ViewObject = $self->viewObject;
        };
        if (ExistsError()) {
            # use Unity shop as redirect object in case the view object cannot be found
            $ViewObject = $Site;
        }

        $self->redirect($ViewObject->get('WebUrlUnitySSL'));
    }
}

1;
